// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 18th April 2006
//
// Purpose:       Demo shows how to use HEL's Background Text System
//                to print text on a tile based background.
//
//                It uses a palette generated by hel/tools/win32/paletti.exe
//								and performs the colorcycle through Horizontal-Blank.
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------

// The background number the 
// background Text System uses.
#define BGNO        0


// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------

// A buffer of memory which the Background Text
// System uses to manage internal states.
// It must not be modified from outside as long as
// any Background Text System is running.
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_BgTextSystemMemory[HEL_SUBSYSTEM_BGTEXT_REQUIREDMEMORY];

// g_CharLUT will hold offsets to each character
// in the map data. This array basically functions
// as a lookup table. It must be 256 elements big,
// which equals the maximum number of a characters
// in ASCII (0..255).
// This buffer is only neccessary when you pass
// BGTEXT_FLAGS_GENERATELUT to hel_BgTextCreate.
// Please see documentation for details.
u16 ATTR_EWRAM g_CharLUT[256];

// That's the order of characters how they
// appear on the source graphic.
// See hel/demos/sharedmedia/charset8x8b.bmp
const char CHARORDER[] =
    " abcdefghijklnmo"\
    "pqrstuvwxyz12345"\
    "67890?!.,+=/O*#-"\
    "><\"~";

// Amount of rows in  g_TextTable
#define TEXTTABLE_COUNT   11

// Notice the "const char* const" here. If you
// don't do it this way, the text does not make
// it into ROM!
const char* const g_TextTable[TEXTTABLE_COUNT] =
{
  "so many pretty parts and",
  "no pretty wholes.",
  "",
  "i need more parts.",
  "",
  "i told you to face the",
  "goddamn wall!",
  "",
  "i have always said,",
  "\"if you can't find a",
  "friend, make one.\""
};

// Gets increased every VBL, to
// perform the color cycle animation.
u32 ATTR_EWRAM g_ColorCycle=0;


// -----------------------------------------------------------------------------
// Function Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);
void ATTR_FASTFUNC HBLInterruptHandler(void);



// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    ham_SetBgMode(0);

    // Initialize the Background Text System.
    // The g_BgTextSystemMemory array is used by the
    // Background Text System to manage internal states
    // and must not be modified from outside as long as
    // any Background Text System is running.
    hel_BgTextInit((void*)g_BgTextSystemMemory);

    // Initialize the tileset and an empty
    // mapset using regular HAMlib functions
    ham_bg[BGNO].ti = ham_InitTileSet(ResData(RES_CHARSET8X8B_RAW), RES_CHARSET8X8B_RAW_SIZE16, COLORS_16, TRUE);
    ham_bg[BGNO].mi = ham_InitMapEmptySet(MAP_SIZE_32X32, FALSE);
    
    // Create a Background Text System
    hel_BgTextCreate(
        BGNO,   // BgNo
        1,      // Width of one character specified in tiles
        1,      // Height of one character specified in tiles
        ResData(RES_CHARSET8X8B_MAP),    // Pointer to character map
        CHARORDER,      // Pointer to array that reflects the order of characters on source graphic
        g_CharLUT,      // Pointer to buffer that receives the generated lookup table
        BGTEXT_FLAGS_GENERATELUT);  // Flags that control creation and print behaviour

    
		{
	    int i;
	
	    for(i=0; i<TEXTTABLE_COUNT; i++)
	    {
	      // Print text on background.
	      hel_BgTextPrint(BGNO,
	          1,      // X
	          1+i,    // Y
	          0,      // Flags, must be zero at this time
	          g_TextTable[i]
	          );
	    }
		}


    // Load palette data to Vram
    hel_PalBgLoad16(ResData16(RES_BG15_PAL), 15);
    
    // Initialize and show background
    ham_InitBg(BGNO, TRUE, 0, FALSE);
    
    // Start interrupts
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);
    hel_IntrStartHandler(INT_TYPE_HBL, (void*)HBLInterruptHandler);
    
    // infinite loop to hold
    // the application alive
    for(;;)
    {
      hel_SwiVBlankIntrWait();
    }

    return 0;
}

// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
    g_ColorCycle++;
    
    hel_IntrAcknowledge(INT_TYPE_VBL);
}

// -----------------------------------------------------------------------------
// HBL Interrupt Handler
// -----------------------------------------------------------------------------
void ATTR_FASTFUNC HBLInterruptHandler(void)
{
    // The text graphic uses a 16 color palette, which is stored
    // in Vram at palette bank 15 (last bank). The first index is
    // transparent, the second one is the text color, at index 241
    // We set the text color depending on the scanline (R_VCNT) plus
    // the ColorCyle value which gets increased every VBL to perform
    // the color cycle animation.
    ACCESS_U16(MEM_PAL_BG_COL_256(241)) = ResData16(RES_COLORGRADIENT_PAL)[(R_VCNT + g_ColorCycle)&255];
}

