// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 18th April 2006
//
// Purpose:       Stress-test for Object-System. Creates and deletes
//                several 16 and 256 color objects in one frame.
//
//                The more red the background becomes, the more time elapsed.
//
//                This stress-test greatly shows how fast the object manager
//                from HEL is, plus the advantage of anti screan tearing support.
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

#include "Profiler.h"

// -----------------------------------------------------------------------------
// Global Defines and Macros
// -----------------------------------------------------------------------------

// Enable the following macro to use HAM's
// Object Managemant System and compare performance.
//#define USE_OBJMANAGER_IN_HAM

// Specifies the size of the random value
// lookup table. Must be power of two
#define RANDTABLE_SIZE    512

// Returns a value from the random value
// lookup table.
#define RANDTABLE_GETVALUE		(g_RandTable[(RANDTABLE_SIZE-1) & (g_RandTableSpot++)])

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
u32 ATTR_EWRAM g_nCreateIterations=7;
u32 ATTR_EWRAM g_nDeleteIterations=8;
u16 ATTR_EWRAM g_RandTable[RANDTABLE_SIZE];
u32 ATTR_EWRAM g_RandTableSpot=0;
u8  ATTR_EWRAM ATTR_ALIGNED(4) g_ObjSystemBuffer[HEL_SUBSYSTEM_OBJ_REQUIREDMEMORY];

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void UpdateObjects(void);
void VBLInterruptHandler(void);


#define PROFILE
#ifdef PROFILE
PROFILER_INIT;
#endif


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
	int i;

	ham_Init();
	hel_SysSetPrefetch(TRUE);
	ham_SetBgMode(0);
	hel_PadInit();
	ham_InitText(0);

#ifdef USE_OBJMANAGER_IN_HAM
	ham_InitObj();
#else
	// Initialize Object-System
	hel_ObjInit(g_ObjSystemBuffer);
#endif

	// Create a lookup table with
	// precomputed random values to
	// gain performance. rand() = slow!
	for(i=0; i<RANDTABLE_SIZE; i++)
	{
		g_RandTable[i] = RANDOM(0xffff);
	}

	// load object palette
	hel_PalObjLoad256(ResData16(RES_OBJECT_PAL));
	hel_PalObjLoad16(ResData16(RES_FLOWER_PAL), 15);

	hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);


	for(;;)
	{
		hel_PadCapture();

		if(hel_PadQuery()->Pressed.A)
		{
#ifdef USE_OBJMANAGER_IN_HAM
			ham_ResetObj();
#else
			hel_ObjQuit();
			hel_ObjInit(g_ObjSystemBuffer);
#endif
		}



#ifdef PROFILE
		PROFILER_START;
#endif

		ACCESS_U16(MEM_PAL_BG) = RGB_SET(16,0,0);

		// update object graphics
		UpdateObjects();

		ACCESS_U16(MEM_PAL_BG) = RGB_SET(0,16,0);

#ifdef PROFILE
		PROFILER_END;
#endif


#ifdef USE_OBJMANAGER_IN_HAM
		ham_DrawText(1, 1, "%d objs free   ", ham_GetObjAvailable());
#else
		ham_DrawText(1, 1, "%d objs free   ", hel_ObjGetFreeSlots());
#endif

		hel_SwiVBlankIntrWait();
	}

	return 0;
}



// -----------------------------------------------------------------------------
// Update Objects, load new graphic into Vram.
// -----------------------------------------------------------------------------
void UpdateObjects(void)
{
	int i;

	for(i=0; i<g_nDeleteIterations; i++)
	{
		THandle Handle = RANDTABLE_GETVALUE & 127;

#ifdef USE_OBJMANAGER_IN_HAM
		if(ham_obj[Handle].taken)
		{
			ham_DeleteObj(Handle);
		}
#else
		if(hel_ObjExists(Handle))
		{
			hel_ObjDelete(Handle);
		}
#endif
	}


	for(i=0; i<g_nCreateIterations; i++)
	{
		u32 ColMode = RANDTABLE_GETVALUE & 1;

#ifdef USE_OBJMANAGER_IN_HAM
		if(ham_GetObjAvailable() > 0)
		{
			ham_CreateObj
					(
						ColMode?ResData(RES_HEART_RAW):ResData(RES_FLOWER_RAW), // Source graphic
						OBJ_SHAPE_SQUARE,     // Object Shape
						1,                    // Object Size
						OBJ_MODE_NORMAL,      // Object Mode
						ColMode,              // Color Mode
						ColMode?0:15,
						FALSE,
						FALSE,
						FALSE,
						1,
						FALSE,
						RANDTABLE_GETVALUE % 226,
						RANDTABLE_GETVALUE % 146
					);
		}
#else
		if(hel_ObjGetFreeSlots() > 0)
		{
			hel_ObjCreate
					(
						ColMode?ResData(RES_HEART_RAW):ResData(RES_FLOWER_RAW), // Source graphic
						OBJ_SHAPE_SQUARE,     // Object Shape
						1,                    // Object Size
						OBJ_MODE_NORMAL,      // Object Mode
						ColMode,              // Color Mode
						ColMode?0:15,
						FALSE,
						FALSE,
						FALSE,
						1,
						FALSE,
						RANDTABLE_GETVALUE % 226,
						RANDTABLE_GETVALUE % 146
					);
		}
#endif
	}
}

// -----------------------------------------------------------------------------
// VBL Interupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
#ifdef USE_OBJMANAGER_IN_HAM
	ham_CopyObjToOAM();
#else
	// Transmit object attributes to hardware
	hel_ObjTransmit();
#endif

	hel_IntrAcknowledge(INT_TYPE_VBL);
}




