// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 12th August 2006
//
// Purpose:       Stress-test for HEL Library. Creates and deletes
//                several objects per frame, alpha-blending enabled and updated,
//                two windows active, blending between two 256 color palettes as
//                well as a map with dynamic tile reloading and wrapping.
//
// Note:
//                If you can't compile the project because of an error
//                the Krawall sound converter creates, try to set
//                "KRAWALL_IS_REGISTERED" in the project makefile to "no"
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"
#include "lookups.h"
#include "Profiler.h"
#include "krawall.h"
#include "modules.h"


// -----------------------------------------------------------------------------
// Global Defines and Macros
// -----------------------------------------------------------------------------

// Specifies the size of the random value
// lookup table. Must be power of two
#define RANDTABLE_SIZE    512

// Returns a value from the random value
// lookup table.
#define RANDTABLE_GETVALUE		g_RandTable[(RANDTABLE_SIZE-1) & (g_RandTableSpot++)]

// Want to enable/disable music?
//#define MUSIC_ENABLED

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
u8  ATTR_EWRAM ATTR_ALIGNED(4) g_MapSystemBuffer[HEL_SUBSYSTEM_MAP_REQUIREDMEMORY];
u8  ATTR_EWRAM ATTR_ALIGNED(4) g_TileSystemBuffer[HEL_SUBSYSTEM_TILE_REQUIREDMEMORY];
u8  ATTR_EWRAM ATTR_ALIGNED(4) g_ObjSystemBuffer[HEL_SUBSYSTEM_OBJ_REQUIREDMEMORY];

u32 ATTR_EWRAM g_nCreateIterations=7;
u32 ATTR_EWRAM g_nDeleteIterations=8;
u16 ATTR_EWRAM g_RandTable[RANDTABLE_SIZE];
u32 ATTR_EWRAM g_RandTableSpot=0;


#define NUMBER_TILES_RAM    (31*21) // Screen width and height in tiles, both extended by one
#define NUMBER_TILES_ROM    (7438)  // Background map has this amount of unique tiles
#define VAMPIRE_WIDTH       (800/8) // Background map width in tiles
#define VAMPIRE_HEIGHT      (600/8) // Background map height in tiles

u16 ATTR_EWRAM g_BufferRom[NUMBER_TILES_ROM];
u16 ATTR_EWRAM g_BufferRam[NUMBER_TILES_RAM*3];

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void UpdateObjects(void);
void UpdateWindows(void);
void UpdateAlphaLevel(void);
void UpdatePalettes(void);
void VBLInterruptHandler(void);
void ScrollMap();


//#define PROFILE

#ifdef PROFILE
PROFILER_INIT;
#endif

// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    int i;
    
    ham_Init();
    ham_SetBgMode(0);
    
#ifdef PROFILE
    ham_InitText(1);
    ham_SetTextCol(252, 0);
    for(i=0; i<20; i++)
      ham_DrawText(0,i,"                              ");
#endif

    hel_SysSetPrefetch(TRUE);
    
    // Initialize Sub-Systems
    hel_MapInit(g_MapSystemBuffer);
    hel_TileInit(g_TileSystemBuffer);
    hel_ObjInit(g_ObjSystemBuffer);
    hel_WinInit();
    hel_PadInit();
    hel_FxInit();
    
  	// Create a lookup table with
  	// precomputed random values to
  	// gain performance. rand() = slow!
  	for(i=0; i<RANDTABLE_SIZE; i++)
  	{
  		g_RandTable[i] = RANDOM(0xffff);
  	}
	
    // Init Tile-System for Background 0
    hel_TileCreate(0,                // BgNo
                 ResData(RES_VAMPIRE_RAW),    // Source tiledata
                 NUMBER_TILES_ROM, // Number of tiles in ROM
                 g_BufferRom,      // BufferA
                 NUMBER_TILES_RAM, // Maximum number of tiles in RAM
                 g_BufferRam,      // BufferB
                 1,                // Tileset colormode (0=16colors, 1=256colors)
                 0,                // Palettenumber in 16color mode (0..15)
                 TRUE);            // CBB only mode

    // Init Map-System for Background 0
    hel_MapCreate(0,                         // BgNo
                  VAMPIRE_WIDTH,             // MapWidth in tiles
                  VAMPIRE_HEIGHT,            // MapHeight in tiles
                  ResData(RES_VAMPIRE_MAP),  // Source MapData
                  sizeof(u16),               // DataTypeSize
                  (MAP_FLAGS_DEFAULT)| MAP_FLAGS_DYNAMICTILERELOADING); // Flags to control map behaviour

    // Disable Bounds-Checking, which enabled map-wrap-around
    hel_MapSetBounds(0, 0, 0, FIXED_FROMINT(SCREEN_DISPLAY_WIDTH), FIXED_FROMINT(SCREEN_DISPLAY_HEIGHT), MAP_BOUNDS_NONE);
    
    // Load Background Palette
    hel_PalBgLoad256(ResData(RES_VAMPIRE_PAL));
    
    // load object palette
    hel_PalObjLoad256(ResData16(RES_OBJECT_PAL));
    hel_PalObjLoad16(ResData16(RES_FLOWER_PAL), 15);
    
    // enable alpha blending
    hel_FxSetMode(FX_LAYER_OBJ, FX_LAYER_ALL, FX_MODE_ALPHABLEND);
    
    hel_WinCreate(0, 0, 0, 8, 160, WIN_NONE, WIN_ALL | WIN_FX);
    hel_WinCreate(1, 0, 0, 0, 0, WIN_BG0, WIN_ALL);
    
#ifdef MUSIC_ENABLED
    // initialize krawall sound system
    kragInit(KRAG_INIT_STEREO);
    
    // Start Timer 1 for krawall sound system
    hel_IntrStartHandler(INT_TYPE_TIM1, (void*)kradInterrupt);

    // start music
    krapPlay(&mod_acid, KRAP_MODE_LOOP, 0);
#endif

    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);

    // Turn on Background
    ham_InitBg(0, TRUE, 0, FALSE);

    // set bg prio
    hel_BgSetPrio(0, 1);
    
    // infinite loop
    for(;;)
    {

#ifdef PROFILE
PROFILER_START;
#endif

#ifdef MUSIC_ENABLED
        // Update Krawall Sound System
        ACCESS_U16(MEM_PAL_BG) = RGB_SET(0,0,31);
        kramWorker();
#endif


        hel_PadCapture();

        if(hel_PadQuery()->Pressed.A)
        {
          hel_ObjQuit();
          hel_ObjInit(g_ObjSystemBuffer);
        }


        ACCESS_U16(MEM_PAL_BG) = COLOR_CREAM;
        UpdatePalettes();
        
        ACCESS_U16(MEM_PAL_BG) = COLOR_MONEYGREEN;
        UpdateWindows();
        
        ACCESS_U16(MEM_PAL_BG) = COLOR_YELLOW;
        UpdateAlphaLevel();

        ACCESS_U16(MEM_PAL_BG) = COLOR_OLIVE;
        ScrollMap();

        ACCESS_U16(MEM_PAL_BG) = COLOR_FUCHSIA;
        UpdateObjects();
        
        ACCESS_U16(MEM_PAL_BG) = COLOR_LIME;

#ifdef PROFILE
PROFILER_END;
#endif

      hel_SwiVBlankIntrWait();
    }
    
    return 0;
}

void UpdatePalettes(void)
{
   const u32 LUT[64]=
      {
         0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,
         31,30,29,28,27,26,25,24,23,22,21,20,19,18,17,16,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,1
      };
   static u32 Counter=0;
   
   Counter++;
   Counter&=63;
   
   hel_PalBgBlend256(ResData(RES_VAMPIRE_PAL), ResData(RES_VAMPIRE2_PAL), LUT[Counter]);
}


void UpdateWindows(void)
{
   static int SinCounter=0;
   
   SinCounter++;
   SinCounter%=300;
   
   hel_WinSetRegion
      (
         1,
         WindowSinLUT[SinCounter], WindowSinLUT[(SinCounter*2)%300],
         WindowSinLUT[SinCounter]+64, WindowSinLUT[(SinCounter*2)%300]+64
      );
}

// -----------------------------------------------------------------------------
// Scrolls the map
// -----------------------------------------------------------------------------
void ScrollMap()
{
   #define SCROLL_SPEED_X     FIXED_FROMFLOAT(3.0f)
   #define SCROLL_SPEED_Y     FIXED_FROMFLOAT(1.5f)

   // Now Scroll the map!
   hel_MapScroll(0, SCROLL_SPEED_X, SCROLL_SPEED_Y);
}


// -----------------------------------------------------------------------------
// Update Objects, load new graphic into Vram.
// -----------------------------------------------------------------------------
void UpdateObjects(void)
{
   int i;
   
   for(i=0; i<g_nDeleteIterations; i++)
   {
      THandle Handle = RANDTABLE_GETVALUE & 127;

      if(hel_ObjExists(Handle))
      {
         hel_ObjDelete(Handle);
      }
   }


   for(i=0; i<g_nCreateIterations; i++)
   {
      u32 ColMode = RANDTABLE_GETVALUE & 1;
      
      if(hel_ObjGetFreeSlots() > 0)
      {
         hel_ObjCreate(
                        ColMode?ResData(RES_HEART_RAW):ResData(RES_FLOWER_RAW), // Sourcegraphic
                        OBJ_SHAPE_SQUARE,     // Object Shape
                        1,                    // Object Size
                        OBJ_MODE_NORMAL,      // Object Mode
                        ColMode,              // Color Mode
                        ColMode?0:15,
                        FALSE,
                        FALSE,
                        FALSE,
                        1,
                        FALSE,
                        RANDTABLE_GETVALUE % 226,
                        RANDTABLE_GETVALUE % 146
                      );
      }
   }
}

// -----------------------------------------------------------------------------
// Update Alphablending Level
// -----------------------------------------------------------------------------
void UpdateAlphaLevel(void)
{
   static u8 Intensity=0;
   const u8 LUT[32]={0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1};
   
   Intensity++;
   Intensity&=31;
   
   hel_FxSetAlphaLevel(LUT[Intensity], 16-LUT[Intensity]);
}

// -----------------------------------------------------------------------------
// VBL Interupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
   // Transmit attributes to hardware
   hel_ObjTransmit();
   hel_MapTransmit();
   hel_FxTransmit();
   hel_WinTransmit();
          
   // acknowlede interrupt
   hel_IntrAcknowledge(INT_TYPE_VBL);
}

