// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 18th April 2006
//
// Purpose:       Demo shows how to use the map-system with dynamic tile
//                reloading enabled as well as animated tiles.
//
//                You should have read following samples before:
//                   Map
//                   MapDynamicTileReloading
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------
#define NUMBER_TILES_ROM    (53)
#define NUMBER_TILES_RAM    (53)

// Size of map in tiles
#define WORLD_WIDTH         (1024/8)
#define WORLD_HEIGHT        (256/8)

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
u32 ATTR_EWRAM g_TotalFrames=0;
u32 ATTR_EWRAM g_HeartFrames=0;
u16 ATTR_EWRAM g_BufferRom[NUMBER_TILES_ROM];
u16 ATTR_EWRAM g_BufferRam[NUMBER_TILES_RAM*3];
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_MapSystemBuffer[HEL_SUBSYSTEM_MAP_REQUIREDMEMORY];
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_TileSystemBuffer[HEL_SUBSYSTEM_TILE_REQUIREDMEMORY];

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void AnimateHeartGraphic(void);
void ScrollMap();
void VBLInterruptHandler(void);

// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    hel_PadInit();
    
    // initialize Map and Tile-System
    hel_MapInit(g_MapSystemBuffer);
    hel_TileInit(g_TileSystemBuffer);
    
    ham_SetBgMode(0);

    // Init Tile-System for Background 0
    hel_TileCreate(0,                   // BgNo
                 ResData(RES_WORLD3_LAYER0_RAW), // Source tiledata
                 NUMBER_TILES_ROM,    // Number of tiles in ROM
                 g_BufferRom,         // BufferA
                 NUMBER_TILES_RAM,    // Maximum number of tiles in RAM
                 g_BufferRam,         // BufferB
                 1,                   // Tileset colormode (0=16colors, 1=256colors)
                 0,                   // Palettenumber in 16color mode (0..15)
                 TRUE);               // CBB only mode


    // Init Map-System for Background 0
    hel_MapCreate(0,                  // BgNo
                  WORLD_WIDTH,        // MapWidth in tiles
                  WORLD_HEIGHT,       // MapHeight in tiles
                  ResData(RES_WORLD3_LAYER0_MAP), // Source MapData
                  sizeof(u16),              // DataTypeSize
                  MAP_FLAGS_DEFAULT | MAP_FLAGS_DYNAMICTILERELOADING);// Flags to control map behaviour

    // Load Background Palette
    hel_PalBgLoad256(ResData(RES_BG_PAL));

    // Turn on Background
    ham_InitBg(0, TRUE, 0, FALSE);

    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);



    // Infinite loop
    for(;;)
    {
      hel_PadCapture();
      ScrollMap();
      AnimateHeartGraphic();
      
      hel_SwiVBlankIntrWait();
    }
    
    return 0;
}

// -----------------------------------------------------------------------------
// Animate heart graphic
//
// Check if the tile is currently loaded. If it is not, it is not visible
// and can be skipped. The worldmap has in the top left corner the heart, which
// we use as reference tilenumber, because the tilenumber is for every heart the same.
// -----------------------------------------------------------------------------
void AnimateHeartGraphic(void)
{
  // Get number of tile specified by X and Y from our worldmap.
  #define GetWorldMapTile(X,Y)    ResData16(RES_WORLD3_LAYER0_MAP)[((Y)*WORLD_WIDTH)+(X)]
  
  // Get memory address to one of the heart tiles.
  // Check "demos\SharedMedia\gfx\heart.bmp" to see how the graphic is
  // organised. The graphic has 2 tiles on X and 4 tiles on Y.
  // We do Y*128, because 1 tile in 256 colors is 64bytes and
  // because we want to skip 2 tiles (remember, 2 tiles on X) we
  // multiply by 128!
  #define GetHeartTile(X,Y)       ResData8(RES_HEART_RAW)[((X)*64)+((Y)*128)]
  
  // The rownumber where each heart starts
  u32 RowNo = (g_HeartFrames&1)*2;
  
  
   // Is the top-left tile of the heart loaded?
  if(hel_TileIsGraphicLoaded(0, GetWorldMapTile(0,0)))
  {
    // YES, it is! So now reload the graphic!
    hel_TileReloadGraphic(0, GetWorldMapTile(0,0), &GetHeartTile(0, RowNo));
  }

  // Is the top-right tile of the heart loaded?
  if(hel_TileIsGraphicLoaded(0, GetWorldMapTile(1,0)))
  {
    hel_TileReloadGraphic(0, GetWorldMapTile(1,0), &GetHeartTile(1, RowNo));
  }

   // Is the bottom-left tile of the heart loaded?
  if(hel_TileIsGraphicLoaded(0, GetWorldMapTile(0,1)))
  {
    hel_TileReloadGraphic(0, GetWorldMapTile(0,1), &GetHeartTile(0, RowNo+1));
  }

   // Is the bottom-right tile of the heart loaded?
  if(hel_TileIsGraphicLoaded(0, GetWorldMapTile(1,1)))
  {
    hel_TileReloadGraphic(0, GetWorldMapTile(1,1), &GetHeartTile(1, RowNo+1));
  }
}

// -----------------------------------------------------------------------------
// Query Pad and scroll map depending on current pad input
// -----------------------------------------------------------------------------
void ScrollMap()
{
   #define SCROLL_SPEED_X     FIXED_FROMFLOAT(2.0f)
   #define SCROLL_SPEED_Y     FIXED_FROMFLOAT(2.0f)

   TPointSfp32 ScrollBy={0, 0};

   // Query Pad to see in what direction
   // the map should be scrolled
   if(hel_PadQuery()->Held.Left)
   {
      ScrollBy.X -= SCROLL_SPEED_X;
   }
   else
   if(hel_PadQuery()->Held.Right)
   {
      ScrollBy.X += SCROLL_SPEED_X;
   }

   if(hel_PadQuery()->Held.Down)
   {
      ScrollBy.Y += SCROLL_SPEED_Y;
   }
   else
   if(hel_PadQuery()->Held.Up)
   {
      ScrollBy.Y -= SCROLL_SPEED_Y;
   }

   // Now Scroll the map!
   hel_MapScroll(0, ScrollBy.X, ScrollBy.Y);
}


// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
	g_TotalFrames++;

	// Only increase g_HeartFrames every 16th frame
	if(!(g_TotalFrames&15))
    g_HeartFrames++;
    
  hel_IntrAcknowledge(INT_TYPE_VBL);
}

