//////////////////////////////////////////////////////////////////////////
//
//	HEL Library (c) 2003-2006 by Peter Schraut
//	http://www.console-dev.de
//
// -----------------------------------------------------------------------
//
//	HEL Library is distributed as freeware.
//
//	You are free to use HEL Library as part of your program for any purpose including 
//	freeware, shareware and commercial programs, provided some credit is given to the 
//	author. Credits must be given in the credits-screen of your program. If your
//	program has no credits-screen, credits must be provided in a readme file and if
//	available, also in the program manual/handbook.
//	
//	At least one of the following credit texts must be used.
//	If you want to include a different text, it must be first accepted by the author. 	
//	
//		- HEL Library by Peter Schraut
//		- Additional code by Peter Schraut
//
//
//	The origin of this software must not be misrepresented; you must not claim your 
//	authorship. All redistributions must retain the original copyright notice and web 
//	site addresses.
//
//	Commercial redistribution of the library is allowed only with an explicit written
//	permission from the author.
//
//	If you finished a commercial game where HEL Library is used, the author must
//	be informed in what game it is used. A free copy of the final product is appreaciated,
//	but not required.
//
//	This software is provided 'as-is', without warranty of any kind, either expressed or 
//	implied. In no event shall the author be held liable for any damages arising from the 
//	use of this software.
//
//////////////////////////////////////////////////////////////////////////

#ifndef HEL2_H
#define HEL2_H

#include <mygba.h>
#include <stdio.h>
#include <stdarg.h>
#include <malloc.h>


//////////////////////////////////////////////////////////////////////////
// HEL Library version defines
//////////////////////////////////////////////////////////////////////////

// When generating the api documentation using DoxyGen, 
// it always must scan the sourcecodes in debug mode.
// That's because I only documented the functions and 
// since a few of them get replaced with macros in
// release mode, DoxyGen would miss them!
#ifdef DOXYGEN_SHOULD_SKIP_THIS
#		define HEL_DEBUG
#endif

#define HEL_VERSION_MAJOR		2
#define HEL_VERSION_MINOR		0
#define HEL_VERSION_PATCH		0
#define HEL_VERSION_NAME		"HEL Library 2.0.0"

#define GCC_VERSION_MAJOR		__GNUC__
#define GCC_VERSION_MINOR		__GNUC_MINOR__


//////////////////////////////////////////////////////////////////////////
// HEL Library prerequisites
//////////////////////////////////////////////////////////////////////////

#if defined(HEL_DEBUG) || defined(HEL_CHECKED)
#	define HEL_DEBUG_MESSAGE_ENABLED	1
#	ifdef HEL_DEBUG_CALLSTACKTRACE
#		define HEL_DEBUG_CALLSTACKTRACE_ENABLED	1
#	endif
#	define HEL_DEBUG_MSG(txt, args...)	__hel_DebugMessageFunc(txt, ##args)
#	define HEL_DEBUG_BRK                __asm ("mov r11, r11 @ no$gba breakpoint\n")
#	define HEL_INLINE
#else
#	define HEL_DEBUG_MSG(txt, args...)
#	define HEL_DEBUG_BRK
#	define HEL_INLINE					inline
#endif

#ifdef __cplusplus
#	define HEL_API	extern "C"
#else
#	define HEL_API	extern
#endif


//////////////////////////////////////////////////////////////////////////
// HEL Library Sub-System memory service macros
//////////////////////////////////////////////////////////////////////////

/*!
\ingroup helObj
\brief  Required memory for the Object System
\hideinitializer

	This define represents the amount of memory, specified in bytes, the 
	Object System from HEL requires to manage its internal states.
	When you initialize the Object System, you must pass a buffer which 
	equals the size of #HEL_SUBSYSTEM_OBJ_REQUIREDMEMORY.

\sa
	#hel_ObjInit
*/
#define HEL_SUBSYSTEM_OBJ_REQUIREDMEMORY			((sizeof(TObjInfo)*128) + (sizeof(u8)*128) + (sizeof(u16)*1024) + (sizeof(u32)*4))

/*!
\ingroup helObjRotScale
\brief  Required memory for the Object Rotation/Scaling System
\hideinitializer

	This define represents the amount of memory, specified in bytes, the 
	Object Rotation/Scaling System from HEL requires to manage its internal states.
	When you initialize the Object Rotation/Scaling System, you must pass a buffer 
	which equals the size of #HEL_SUBSYSTEM_OBJROTSCALE_REQUIREDMEMORY.

\sa
	#hel_ObjRotScaleInit
*/
#define HEL_SUBSYSTEM_OBJROTSCALE_REQUIREDMEMORY	((sizeof(u8)*32) + (sizeof(THandle)*32) + (sizeof(u16)*32*4) + (sizeof(u32)*4) + sizeof(u8))

/*!
\ingroup helMap
\brief  Required memory for the Map System
\hideinitializer

	This define represents the amount of memory, specified in bytes, the 
	Map System from HEL requires to manage its internal states.
	When you initialize the Map System, you must pass a buffer which 
	equals the size of #HEL_SUBSYSTEM_MAP_REQUIREDMEMORY.

\sa
	#hel_MapInit
*/
#define HEL_SUBSYSTEM_MAP_REQUIREDMEMORY					((sizeof(TMapInfo)*5) + sizeof(TMapDrawInfo) + (sizeof(u32)*2))

/*!
\ingroup helTile
\brief  Required memory for the Tile System
\hideinitializer

	This define represents the amount of memory, specified in bytes, the 
	Tile System from HEL requires to manage its internal states.
	When you initialize the Tile System, you must pass a buffer which 
	equals the size of #HEL_SUBSYSTEM_TILE_REQUIREDMEMORY.

\sa
	#hel_TileInit
*/
#define HEL_SUBSYSTEM_TILE_REQUIREDMEMORY					(((sizeof(TTileInfo)+sizeof(TTileSystemInfo))*4) + (sizeof(u32)*2))

/*!
\ingroup helBgText
\brief  Required memory for the Background Text System
\hideinitializer

	This define represents the amount of memory, specified in bytes, the 
	Background Text System from HEL requires to manage its internal states.
	When you initialize the Background Text System, you must pass a buffer which 
	equals the size of #HEL_SUBSYSTEM_BGTEXT_REQUIREDMEMORY.

\sa
	#hel_TileInit
*/
#define HEL_SUBSYSTEM_BGTEXT_REQUIREDMEMORY				((sizeof(TBgTextInfo)*4) + (sizeof(u32)*2))

/*!
\ingroup helSplash
\brief  Required memory for the Splash-Screen System
\hideinitializer

	This define represents the amount of memory, specified in bytes, the 
	Splash-Screen System from HEL requires to manage its internal states.
	When you initialize the Splash-Screen System, you must pass a buffer which 
	equals the size of #HEL_SUBSYSTEM_SPLASH_REQUIREDMEMORY.

\sa
	#hel_SplashInit
*/
#define HEL_SUBSYSTEM_SPLASH_REQUIREDMEMORY		(sizeof(u16)*512)


//////////////////////////////////////////////////////////////////////////
// Generic Structures and Types
//////////////////////////////////////////////////////////////////////////

typedef struct _TPoint8
{
	s8 X;
	s8 Y;
}TPoint8, *PPoint8;

typedef struct _TPointU8
{
	u8 X;
	u8 Y;
}TPointU8, *PPointU8;

typedef struct _TPoint16
{
	s16 X;
	s16 Y;
}TPoint16, *PPoint16;

typedef struct _TPointU16
{
	u16 X;
	u16 Y;
}TPointU16, *PPointU16;

typedef struct _TPoint32
{
	s32 X;
	s32 Y;
}TPoint32, *PPoint32;

typedef struct _TPointU32
{
	u32 X;
	u32 Y;
}TPointU32, *PPointU32;

typedef struct _TPointSfp16
{
	sfp16 X;
	sfp16 Y;
}TPointSfp16, *PPointSfp16;

typedef struct _TPointSfp32
{
	sfp32 X;
	sfp32 Y;
}TPointSfp32, *PPointSfp32;


typedef struct _TRect8
{
	s8 Left;
	s8 Top;
	s8 Right;
	s8 Bottom;
}TRect8, *PRect8;

typedef struct _TRect16
{
	s16 Left;
	s16 Top;
	s16 Right;
	s16 Bottom;
}TRect16, *PRect16;

typedef struct _TRect32
{
	s32 Left;
	s32 Top;
	s32 Right;
	s32 Bottom;
}TRect32, *PRect32;

typedef struct _TRectSfp16
{
	sfp16 Left;
	sfp16 Top;
	sfp16 Right;
	sfp16 Bottom;
}TRectSfp16, *PRectSfp16;

typedef struct _TRectSfp32
{
	sfp32 Left;
	sfp32 Top;
	sfp32 Right;
	sfp32 Bottom;
}TRectSfp32, *PRectSfp32;


typedef u8 THandle;


#ifndef __cplusplus
#	define false		0
#	define true			(!false)
#	ifndef bool
#		define bool		u8
#	endif // ifndef bool
#	ifndef null
#		define null		NULL
#	endif // ifndef null
#endif // #ifndef __cplusplus



//////////////////////////////////////////////////////////////////////////
// Generic Helper Macros
//////////////////////////////////////////////////////////////////////////
#define ODD(v)                         ((v)&1)
#define BIT(n)                         (1<<(n))
#define BIT_NOT(n)                     (~BIT(n))
#define BIT_IS_SET(v,b)                ((((v) >> (b)) & 1))
#define MIN(a,b)                       (((a)<(b))?(a):(b))
#define MID(a,b,c)                     (MAX(MIN((a),(b)),MIN((a),(c)))
#define MAX(a,b)                       (((a)>(b))?(a):(b))
#define LIMIT(Value,ValueMin,ValueMax) (MIN(MAX((Value), (ValueMin)), (ValueMax)))
#define SGN(v)                         (((v) < 0)?-1:(((v) > 0)?(+1):(0)))
#define ABS(v)                         (((v) > 0)?(v):(-(v)))
#define ASSIGNED(p)                    (NULL != (p))
#define ACCESS_U8(v)                   *(volatile u8*)(v)
#define ACCESS_U16(v)                  *(volatile u16*)(v)
#define ACCESS_U32(v)                  *(volatile u32*)(v)
#define RANDOM(n)                      (rand()%(n))
#define SIN(n)                         (SIN_SFP32[(u32)(n)%360])
#define COS(n)                         (COS_SFP32[(u32)(n)%360])

#define ADDR(p)							((u32)(p))
#define ADDR_WORDALIGNED(p)				(((ADDR(p)&3))==0)
#define ADDR_HALFWORDALIGNED(p)			(((ADDR(p)&1))==0)
#define HANDLE_INVALID					(0xff)
#define HANDLE_ISVALID(h)				((h) < 128)


//////////////////////////////////////////////////////////////////////////
// GCC special attributes
//////////////////////////////////////////////////////////////////////////

/*!
\defgroup helGccAttr Compiler special attribute macros
*/

/*!
\ingroup helGccAttr
\hideinitializer
\brief   Align variable or structure field

	#ATTR_ALIGNED specifies a minimum alignment for the variable or structure field,
	measured in bytes. For example, the declaration:
\code
int ATTR_ALIGNED(16) Energy;
\endcode
	causes the compiler to allocate the variable \p Energy on a 16-byte boundary.

\param[in]
	n						Alignment measured in bytes.
*/
#define ATTR_ALIGNED(n)             __attribute__ ((aligned(n)))
#define ATTR_PACKED                 __attribute__ ((packed))
#define ATTR_SECTION(name)          __attribute__ ((section(name)))
#define ATTR_DEPRECATED             __attribute__ ((deprecated))
#define ATTR_NOINLINE               __attribute__ ((noinline))
#define ATTR_FORCEINLINE            __attribute__ ((always_inline))
#define ATTR_USED                   __attribute__ ((used))
#define ATTR_WEAK                   __attribute__ ((weak))
#define ATTR_NORETURN               __attribute__ ((noreturn))
#define ATTR_PURE                   __attribute__ ((pure))
#define ATTR_UNUSED                 __attribute__ ((unused))
#define ATTR_COMMON                 __attribute__ ((common))
#define ATTR_NOCOMMON               __attribute__ ((nocommon))
#define ATTR_NOINSTRUMENT			__attribute__ ((no_instrument_function))


/*!
\ingroup helGccAttr
\hideinitializer
\brief   Store memory in EWRAM

	The #ATTR_EWRAM macro can be used to store variables
	in External Work RAM.

\code
u32 ATTR_EWRAM Buffer[64];
\endcode
*/
#define ATTR_EWRAM		ATTR_SECTION(".ewram") ATTR_NOCOMMON

/*!
\ingroup helGccAttr
\hideinitializer
\brief   Store function in IWRAM

	The #ATTR_FASTFUNC macro can be used to store a
	function in Internal Work RAM.

\code
void ATTR_FASTFUNC SomeFunction(void);

void ATTR_FASTFUNC SomeFunction(void)
{
	// fancy code here
}
\endcode
*/
#define ATTR_FASTFUNC	__attribute__ ((section (".iwram"), long_call))

#if (GCC_VERSION_MAJOR >= 4)
#		define ATTR_WARNUNUSEDRESULT	__attribute__ ((warn_unused_result))
#else
#		define ATTR_WARNUNUSEDRESULT
#endif


//////////////////////////////////////////////////////////////////////////
// GCC built-in functions
//////////////////////////////////////////////////////////////////////////
#define BUILTIN_OFFSETOF(_Type, _Member)	__builtin_offsetof (_Type, _Member)


//////////////////////////////////////////////////////////////////////////
// Generic Defines
//////////////////////////////////////////////////////////////////////////
#define SCREEN_DISPLAY_WIDTH	240
#define SCREEN_DISPLAY_HEIGHT	160

#define COLORS_16	0
#define COLORS_256	1


//////////////////////////////////////////////////////////////////////////
// Fixed Point macros
//
// Used format is 1.19.12
//////////////////////////////////////////////////////////////////////////

/*!
\ingroup PageFixedPoint
\hideinitializer
\brief  \p 1 in fixed point 1.19.12

	The #FIXED_ONE define represents \p 1 (One) in fixed point 1.19.12 format.
*/
#define FIXED_ONE	0x1000L

/*!
\ingroup PageFixedPoint
\hideinitializer
\brief  \p PI in fixed point 1.19.12

	The #FIXED_PI define represents \p PI (3.14159) in fixed point 1.19.12 format.
*/
#define FIXED_PI	FIXED_FROMFLOAT(3.141592653589f)

/*!
\ingroup PageFixedPoint
\hideinitializer
\brief  Convert a float value into fixed point 1.19.12 format
\note		This macro produces slower code than #FIXED_FROMINT
*/
#define FIXED_FROMFLOAT(Value)	((sfp32)((Value)*FIXED_ONE))

/*!
\ingroup PageFixedPoint
\hideinitializer
\brief  Convert an integer into fixed point 1.19.12 format

	The #FIXED_FROMINT macro generates a 1.19.12 fixed point number
	from the number specified by \c Value.

\sa
	#FIXED_TOINT
*/
#define FIXED_FROMINT(Value)	((sfp32)((Value)<<12))

/*!
\ingroup PageFixedPoint
\hideinitializer
\brief  Convert a fixed point 1.19.12 value into an integer

	The #FIXED_TOINT macro generates an integer from the
	fixed point 1.19.12 number specified by \c Value.

\sa
	#FIXED_FROMINT
*/
#define FIXED_TOINT(Value)	((Value)>>12)

/*!
\ingroup PageFixedPoint
\hideinitializer
\brief  Convert a float value into a fixed point 1.19.12

	The #FIXED_TOFLOAT macro generates a float value
	from the fixed point 1.19.12 number specified by \c Value.

\note		This macro should not be used in production code, since
				floating point computations are very expensive on the GBA.
*/
#define FIXED_TOFLOAT(Value)	((float)((Value)/((float)FIXED_ONE)))

/*!
\ingroup PageFixedPoint
\hideinitializer
\brief  Multiply two fixed point 1.19.12 values

	The #FIXED_MUL macro multiplies two fixed point 1.19.12 numbers.
*/
#define FIXED_MUL(A, B)	(sfp32)( (((long long int)(A)*(B))) >> 12 )

/*!
\ingroup PageFixedPoint
\hideinitializer
\brief  Divide two fixed point 1.19.12 values

	The #FIXED_DIV divides the two fixed point 1.19.12 values specified
	by \c Number and \c Denom.

\note
	This macro uses the BIOS divide function. Because of the function call,
	you cannot use it as data initializer.
*/
#define FIXED_DIV(Number, Denom)	hel_SwiDiv(((sfp32)(Number))<<6, ((sfp32)(Denom))>>6)

/*!
\ingroup PageFixedPoint
\hideinitializer
\brief  Truncate a fixed point 1.19.12 value.

	The #FIXED_TRUNC macro generates the number specified by \c Value
	without fraction part. 99.8 becomes 99 and -99.2 becomes -100.
*/
#define FIXED_TRUNC(_Value)	FIXED_FROMINT(FIXED_TOINT(_Value))

//////////////////////////////////////////////////////////////////////////
// Background macros
//////////////////////////////////////////////////////////////////////////
#define R_BGXSCRLX(BgNo)                      ACCESS_16(MEM_IO + 0x10 + ((BgNo)<<2))
#define R_BGXSCRLY(BgNo)                      ACCESS_16(MEM_IO + 0x12 + ((BgNo)<<2))
#define M_BGXSCRLX_SET(BgNo, Value)           R_BGXSCRLX(BgNo) = Value;
#define M_BGXSCRLY_SET(BgNo, Value)           R_BGXSCRLY(BgNo) = Value;
#define M_BGXSCRLXY_SET(BgNo, ValueX, ValueY) do { M_BGXSCRLX_SET(BgNo, ValueX); M_BGXSCRLY_SET(BgNo, ValueY); } while(0)


//////////////////////////////////////////////////////////////////////////
// Object macros
//////////////////////////////////////////////////////////////////////////
#define MEM_OBJ_SLOT(Slot)                    (MEM_OBJ+((Slot)<<5))
#define MEM_OBJ_SLOT_PTR(Slot)                ((u16*)MEM_OBJ_SLOT(Slot))


//////////////////////////////////////////////////////////////////////////
// Color Conversion macros
//////////////////////////////////////////////////////////////////////////
#ifdef RGB_SET
#		undef RGB_SET
#endif

// r,g,b components have to be between 0 and 31
#define RGB_SET(r,g,b)		(((b)<<10)+((g)<<5)+(r))

// extract r,g,b component from bgr555 color
#define RGB_GET_R(rgb)		((rgb) & 0x1f)
#define RGB_GET_G(rgb)		(((rgb) >> 5) & 0x1f)
#define RGB_GET_B(rgb)		(((rgb) >> 10) & 0x1f)



//////////////////////////////////////////////////////////////////////////
// Map System
//
// Implementation: 
//		helMap.c, helMapSetDynamicTileReloading.c,
//		helMapUpdate.c, helMapScroll.c
//////////////////////////////////////////////////////////////////////////

#define BG_VIRTUAL	4

/*!
\ingroup helMapFlags
\hideinitializer
\brief Map Flags Size0

	The \p MAP_FLAGS_SIZE... flags specify the size of the map in Vram. 
	The size depends on the #MAP_FLAGS_ROTATION flag.

Here is an overview:
\verbatim
Flag               Regular     Rotation/Scaling
-----------------------------------------------
MAP_FLAGS_SIZE0    32x32      16x16
MAP_FLAGS_SIZE1    64x32      32x32
MAP_FLAGS_SIZE2    32x64      64x64
MAP_FLAGS_SIZE3    64x64      128x128
\endverbatim
Sizes are specified in tiles, 64x32 means 64 tiles wide and 32 tiles high.
 
	#MAP_FLAGS_SIZE0 is the default flag and will be used if none of the
	\p MAP_FLAGS_SIZE... flags have been specified.

\attention
	This flag must not be changed after Map-System creation.

\note
	Using #MAP_FLAGS_SIZE0 is in most cases the best choice, since it requires the
	smallest amount of memory. However, sometimes you might want to use larger maps,
	which might be necessary when performing mode 7 effects.
*/
#define MAP_FLAGS_SIZE0		0

/*!
\ingroup helMapFlags
\hideinitializer
\brief Map Flags Size1

	Please see #MAP_FLAGS_SIZE0
*/
#define MAP_FLAGS_SIZE1		1

/*!
\ingroup helMapFlags
\hideinitializer
\brief Map Flags Size2

	Please see #MAP_FLAGS_SIZE0
*/
#define MAP_FLAGS_SIZE2		2

/*!
\ingroup helMapFlags
\hideinitializer
\brief Map Flags Size3

	Please see #MAP_FLAGS_SIZE0
*/
#define MAP_FLAGS_SIZE3		3

/*!
\ingroup helMapFlags
\hideinitializer
\brief Control left scrolling behaviour

	If #MAP_FLAGS_LEFT is not included in the map flags, #hel_MapScroll will interpret
	negative \p DeltaX values as zero and therefore the map will not scroll to the left.
*/
#define MAP_FLAGS_LEFT		BIT5

/*!
\ingroup helMapFlags
\hideinitializer
\brief Control right scrolling behaviour

	If #MAP_FLAGS_RIGHT is not included in the map flags, #hel_MapScroll will interpret
	positive \p DeltaX values as zero and therefore the map will not scroll to the right.
*/
#define MAP_FLAGS_RIGHT		BIT6

/*!
\ingroup helMapFlags
\hideinitializer
\brief Control up scrolling behaviour

	If #MAP_FLAGS_UP is not included in the map flags, #hel_MapScroll will interpret
	negative \p DeltaY values as zero and therefore the map will not scroll upwards.
*/
#define MAP_FLAGS_UP		BIT7

/*!
\ingroup helMapFlags
\hideinitializer
\brief Control down scrolling behaviour

	If #MAP_FLAGS_DOWN is not included in the map flags, #hel_MapScroll will interpret
	positive \p DeltaY values as zero and therefore the map will not scroll downwards.
*/
#define MAP_FLAGS_DOWN		BIT8

/*!
\ingroup helMapFlags
\hideinitializer
\brief Transmit position changes immediately

	If #MAP_FLAGS_TRANSMITIMMEDIATELY is set, the Map-System will update the position
	of the map in Vram immediately. #hel_MapTransmit ignores Map-Systems which have
	this flag set.

\note
	If objects do not scroll synchronized with a Map-System, the problem is
	probably the Map-System position it updated immediately. In this case,
	exclude #MAP_FLAGS_TRANSMITIMMEDIATELY and transmit the changes during
	vertical blank using #hel_MapTransmit.
*/
#define MAP_FLAGS_TRANSMITIMMEDIATELY	BIT9

/*!
\ingroup helMapFlags
\hideinitializer
\brief Enable dynamic tile reloading

	The #MAP_FLAGS_DYNAMICTILERELOADING can be used to enable dynamic tile reloading.
	If the Map-System gets associated to a background which tileset has been created 
	using #hel_TileCreate, the #MAP_FLAGS_DYNAMICTILERELOADING must be included in 
	the \c Flags argument when creating a Map-System using #hel_MapCreate.

\attention
	This flag must not be changed after Map-System creation.

\note
	Maps that use the dynamic tile reloading feature must not have H/V flipping and
	palette selection bits set,	because these bits are also used for the tilenumber. 
	To be more precise, bits 0..14 are reserved for the tilenumber. So it supports a 
	maximum of 32767 unique tiles.
*/
#define MAP_FLAGS_DYNAMICTILERELOADING	BIT11

/*!
\ingroup helMapFlags
\hideinitializer
\brief Create a rotation map

	If you want to create a Map-System that supports rotation and scaling, 
	include #MAP_FLAGS_ROTATION in the \p Flags argument when calling #hel_MapCreate. 
	Rotation/Scaling support is necessary for mode 7 effects for example.

	Rotation/Scaling support for tile-backgrounds is only available in mode 1 and 2. 
	It's only available for specific backgrounds. In mode 1, only background 2 supports 
	rotation/scaling, in mode 2 background 2 and 3 support rotation/scaling.

\attention
	This flag must not be changed after Map-System creation.
*/
#define MAP_FLAGS_ROTATION				BIT10

/*!
\ingroup helMapFlags
\hideinitializer
\brief Never transmit position changes

	If #MAP_FLAGS_TRANSMITNEVER is set, the Map-System will never update the position
	of the map in Vram. #hel_MapTransmit ignores Map-Systems which have this flag set.
	You might want to specify this flag if you perform mode 7 effects and need to
	transform each scanline individually.
*/
#define MAP_FLAGS_TRANSMITNEVER			BIT12

/*!
\ingroup helMapFlags
\hideinitializer
\brief Update only the minimum amount of tiles when scrolling

	The #MAP_FLAGS_MINIMALUPDATE flag tells the Map-System to update only map 
	tiles which become visible when the map scrolls. 

	If the Map-System scrolls horizontally, it updates 21 tiles vertically which 
	move into the screen. If the Map-System scrolls vertically, it updates 31 tiles 
	horizontally which move into the screen. These values equal the display dimension 
	in tiles, extended by 1 tile in each direction.

	If a Map-System uses rotation/scaling features, you might want to update an 
	entire row or column of the map in Vram. This is sometimes necessary for mode 
	7 effects. If #MAP_FLAGS_MINIMALUPDATE is not set, the Map-System uses the size 
	of the map in Vram (if the Vram map is 64x64 tiles, it will update 64 tiles 
	vertically and 64 tiles horizontally).

\attention
	This flag must not be changed after Map-System creation.
*/
#define MAP_FLAGS_MINIMALUPDATE		BIT13

/*!
\ingroup helMapFlags
\hideinitializer
\brief Use custom map drawer functions

	The #MAP_FLAGS_CUSTOMMAPDRAWER flag must be specified if you
	want to use custom made map drawer functions.

	This flag works only in conjunction with #hel_MapCreateIndirect.
	The \c pColumnDrawer and \c pRowDrawer members of the #TMapCreateIndirectDesc
	structure must both point to valid map drawer functions.

\sa
	#hel_MapCreateIndirect
*/
#define MAP_FLAGS_CUSTOMMAPDRAWER	BIT14

/*!
\ingroup helMapFlags
\hideinitializer
\brief Default map flags

	The #MAP_FLAGS_DEFAULT represents the default map flags.
*/
#define MAP_FLAGS_DEFAULT			(MAP_FLAGS_LEFT | MAP_FLAGS_RIGHT | MAP_FLAGS_UP | MAP_FLAGS_DOWN | MAP_FLAGS_TRANSMITIMMEDIATELY | MAP_FLAGS_MINIMALUPDATE)

#define MAP_JUMPTO_LEFT				BIT0
#define MAP_JUMPTO_RIGHT			BIT1
#define MAP_JUMPTO_TOP				BIT2
#define MAP_JUMPTO_BOTTOM			BIT3

#define MAP_BOUNDS_NONE				0
#define MAP_BOUNDS_LEFT				BIT0
#define MAP_BOUNDS_RIGHT			BIT1
#define MAP_BOUNDS_TOP				BIT2
#define MAP_BOUNDS_BOTTOM			BIT3
#define MAP_BOUNDS_DEFAULT			(MAP_BOUNDS_LEFT | MAP_BOUNDS_RIGHT | MAP_BOUNDS_TOP | MAP_BOUNDS_BOTTOM)

struct _TMapInfo;		// forward
struct _TMapDrawInfo;	// forward

typedef void (*PMapDrawFunc)(struct _TMapInfo *pMapInfo, struct _TMapDrawInfo *pMapColumnDrawInfo);

// Structure is passed to the
// map-drawing routines to supply
// necessary information for drawing
typedef struct _TMapDrawInfo
{
	void *pSourceCurrentDataAddr;	// Pointer to source element in map
	u32 TargetColumn;				// Horizontal target position in vram
	u32 TargetRow;					// Vertical target position in vram
	u32 TargetScreenBlockBase;		// Address of target ScreenBlockBase
	u32 TargetCharacterBlockBase;	// Address of target CharacterBlockBase
	u32 Quantity;					// Quantity of elements to update (in one direction)
	u32 Flags;						// Flags that control the map-draw behaviour
	TPointU32 Size;					// Width and Height in tiles of mapset in Vram
}TMapDrawInfo, *PMapDrawInfo;


/*!
\ingroup helMap
\brief Describes the map bounds

\sa
	#hel_MapCreateIndirect, #TMapCreateIndirectDesc
*/
typedef struct _TMapBoundsInfo 
{
	sfp32 Left;		//!< Left position in pixels in fixed point format
	sfp32 Top;		//!< Top position in pixels in fixed point format
	sfp32 Right;	//!< Right position in pixels in fixed point format
	sfp32 Bottom;	//!< Bottom position in pixels in fixed point format
	u32 Flags;		//!< Flags that control the bounds behaviour. A combination of the \c MAP_BOUNDS_... flags
}TMapBoundsInfo;
typedef TMapBoundsInfo* PMapBoundsInfo;


/*!
\ingroup helMap
\brief Describes the map size

\sa
	#hel_MapCreateIndirect, #TMapCreateIndirectDesc
*/
typedef struct _TMapSizeInfo
{
	u32 X;		//!< Width in tiles
	u32 Y;		//!< Height in tiles
	u32 Flags;	//!< Size-Flag for hardware map size.
				//!< This can be one of the following values:
				//!<	\li #MAP_FLAGS_SIZE0
				//!<	\li #MAP_FLAGS_SIZE1
				//!<	\li #MAP_FLAGS_SIZE2
				//!<	\li #MAP_FLAGS_SIZE3
} TMapSizeInfo;
typedef TMapSizeInfo*	PMapSizeInfo;


// Holds information about
// an existing Map-System
typedef struct _TMapInfo
{
	u32 BgNo;					// Background Number
	TMapSizeInfo Size;			// Map-Size in tiles
	void *pData;				// Pointer to Map-Data
	u32 DataTypeSize;			// Size of one element in pData
	TPointSfp32 Position;		// Current scroll position in fixed point format
	u32 Flags;					// Flags which specify in what direction the system is allowed to scroll
	PMapDrawFunc pColumnDrawer;	// Pointer to Column-Drawing Function
	PMapDrawFunc pRowDrawer;	// Pointer to Row-Drawing Function
	TPointU32 UpdateQuantity;	// Amount of tiles to update when redrawing a row or column
	TMapBoundsInfo Bounds;		// Map bounds, map can only move within this area
}TMapInfo;
typedef TMapInfo*	PMapInfo;


/*!
\ingroup helMap
\brief Describes the map creation

\sa
	#hel_MapCreateIndirect
*/
typedef struct _TMapCreateIndirectDesc 
{
	u8 BgNo;					//!< Background Number
	TMapSizeInfo Size;			//!< Size information
	const void* pData;			//!< Pointer to Map-Data
	u8 DataTypeSize;			//!< Size of one element in pData
	TPointSfp32 Position;		//!< Position in pixels, specified in fixed point format
	u32 Flags;					//!< Combination of Map Flags, see \ref helMapFlags
	PMapDrawFunc pColumnDrawer;	//!< Pointer to custom map column-drawing function. Necessary only if \c Flags has #MAP_FLAGS_CUSTOMMAPDRAWER set
	PMapDrawFunc pRowDrawer;	//!< Pointer to custom map row-drawing function. Necessary only if \c Flags has #MAP_FLAGS_CUSTOMMAPDRAWER set
	TMapBoundsInfo Bounds;		//!< Map boundary information. Map can only move within this area
} TMapCreateIndirectDesc;
typedef TMapCreateIndirectDesc*	PMapCreateIndirectDesc;


// helMap.c
HEL_API void hel_MapInit(void *pBuffer);
HEL_API void hel_MapQuit(void);
HEL_API void hel_MapCreate(u32 BgNo, u32 Width, u32 Height, const void *pData, u32 DataTypeSize, u32 Flags);
HEL_API void hel_MapCreateIndirect(const TMapCreateIndirectDesc* pDesc);
HEL_API void hel_MapDelete(u32 BgNo);
HEL_API void hel_MapSetPosition(u32 BgNo, sfp32 X, sfp32 Y);
HEL_API void hel_MapGetPosition(u32 BgNo, sfp32 *pX, sfp32 *pY);
HEL_API void hel_MapSetFlags(u32 BgNo, u32 Flags);
HEL_API u32 ATTR_WARNUNUSEDRESULT hel_MapGetFlags(u32 BgNo);
HEL_API void hel_MapRedraw(u32 BgNo);
HEL_API u32 hel_MapScroll(u32 BgNo, sfp32 DeltaX, sfp32 DeltaY);
HEL_API void hel_MapTransmit(void);
HEL_API void hel_MapJumpTo(u32 BgNo, u32 JumpToFlag);
HEL_API void hel_MapSetBounds(u32 BgNo, sfp32 Left, sfp32 Top, sfp32 Right, sfp32 Bottom, u32 Flags);
HEL_API const void ATTR_WARNUNUSEDRESULT *hel_MapGetTilePtrAt(u32 BgNo, sfp32 X, sfp32 Y);
HEL_API u32 ATTR_WARNUNUSEDRESULT hel_MapExists(u32 BgNo);
HEL_API void hel_MapConvertPositionToTiles(u32 BgNo, sfp32 X, sfp32 Y, int *pOutX, int *pOutY);

//////////////////////////////////////////////////////////////////////////
// Map Draw System
//
// Implementation: 
//		helMapLut.c, helMapDrawer.c, helMapDrawer.text.iwram.c
//////////////////////////////////////////////////////////////////////////

#define MAP_DRAWINFOFLAGS_SHOW	BIT0
#define MAP_DRAWINFOFLAGS_HIDE	BIT1



//////////////////////////////////////////////////////////////////////////
// Tile System
//
// Implementation:
//		helTile.c, helTile.text.iwram.c
//////////////////////////////////////////////////////////////////////////

#define TILE_UNUSED			0xffff

typedef struct _TTileInfo
{
	u8 *pTileData;    		// Pointer to source tiledata, usually located in ROM
	u32 NumTilesRom;  		// Number of tiles in source tiledata
	u16 *pRomToRam;   		// Pointer to an allocated buffer of NumTilesRom 16bit words (should be located in EWRAM)
	u32 NumTilesRam;  		// Number of tiles to store in Vram and work with
	u16 *pRamToRom;   		// Pointer to an allocated buffer of NumTilesRam 16bit words (should be located in EWRAM)
	u16 *pRefCount;   		// Pointer to an allocated buffer of NumTilesRam 16bit words (should be located in EWRAM)
	u16 *pTileSlot;   		// Pointer to an allocated buffer of NumTilesRam 16bit words (should be located in EWRAM)
	u32 SlotCounter;  		// Index to the next free slot in pRamToRom
	tile_info *ti;    		// Needed by HAM for tileset managing
	u32 RefCount;     		// Reference Counter. Specifies how often the Tile-Info is used/shared.
	u32 CbbAddr;			// Character Block Base address from tile system
	u32 TileSizeShiftFactor;// Shift factor to get the size of one tile (5 for 4bit, 6 for 8bit)
	u32 TileSizeInBytes;	// Size of one tile in bytes (32 for 4bit, 64 for 8bit)
	u32 TileSizeInHalfWords;// Size of one tile in halfword (16 for 4bit, 32 for 8bit)
	u32 TileSizeInWords;	// Size of one tile in bytes (8 for 4bit, 16 for 8bit)
}TTileInfo, *PTileInfo;

typedef struct _TTileSystemInfo
{
	u32 BgNo;				// Background the TileSystem is assigned to
	u32 PalNo;				// Palette number shifted by 12 to the left, if it's a 16 color background, otherwise set it to zero
	TTileInfo *pTileInfo;	// Pointer to TileInfo
	u32 _pad;				// Padding for power of two alignment
}TTileSystemInfo, *PTileSystemInfo;


#define __hel_TileIsGraphicLoaded(BgNo, RomTileNo)\
	(TILE_UNUSED != __hel_TileInfo[BgNo].pRomToRam[RomTileNo])

#define __hel_TileReloadGraphic16(BgNo, RomTileNo, pSource)\
	do {\
		TTileInfo* pTileInfo=__hel_TileSystemInfo[BgNo].pTileInfo;\
		__hel_DmaCopy16(pTileInfo->CbbAddr + (pTileInfo->pRomToRam[RomTileNo]<<5), pSource, 16);\
	} while(0)

#define __hel_TileReloadGraphic256(BgNo, RomTileNo, pSource)\
	do {\
		TTileInfo* pTileInfo=__hel_TileSystemInfo[BgNo].pTileInfo;\
		__hel_DmaCopy16(pTileInfo->CbbAddr + (pTileInfo->pRomToRam[RomTileNo]<<6), pSource, 32);\
	} while(0)

#define __hel_TileSetMapTile(BgNo, pDest, pSource, Flags)\
	do {\
	TTileSystemInfo* pTileSystem=&__hel_TileSystemInfo[BgNo];\
	if(((Flags) & MAP_DRAWINFOFLAGS_HIDE)) __hel_TileFreeMapTileImplementation(pTileSystem->pTileInfo, pDest);\
	if(((Flags) & MAP_DRAWINFOFLAGS_SHOW)) __hel_TileAllocMapTileImplementation(pTileSystem, pDest, pSource);\
	} while(0)


#ifdef HEL_DEBUG
  HEL_API u32 ATTR_WARNUNUSEDRESULT hel_TileIsGraphicLoaded(u32 BgNo, u32 RomTileNo);
  HEL_API void hel_TileReloadGraphic16(u32 BgNo, u32 RomTileNo, const u8* pSource);
  HEL_API void hel_TileReloadGraphic256(u32 BgNo, u32 RomTileNo, const u8* pSource);
	HEL_API void ATTR_FASTFUNC ATTR_NOINSTRUMENT hel_TileSetMapTile(u32 BgNo, u16 *pDest, const void *pSource, u32 Flags);
#else
	#define hel_TileIsGraphicLoaded(BgNo, RomTileNo)			__hel_TileIsGraphicLoaded(BgNo, RomTileNo)
	#define hel_TileReloadGraphic16(BgNo, RomTileNo, pSource)	__hel_TileReloadGraphic16(BgNo, RomTileNo, pSource)
	#define hel_TileReloadGraphic256(BgNo, RomTileNo, pSource)	__hel_TileReloadGraphic256(BgNo, RomTileNo, pSource)
	#define hel_TileSetMapTile(BgNo, pDest, pSource, Flags)		__hel_TileSetMapTile(BgNo, pDest, pSource, Flags)
#endif

HEL_API void hel_TileInit(void *pBuffer);
HEL_API void hel_TileQuit(void);
HEL_API void hel_TileCreate(u32 BgNo, const u8 *pTileData, u16 NumTilesRom, u16 *pBufferA, u16 NumTilesRam, u16 *pBufferB, u8 ColMode, u8 PalNo, u8 CbbOnlyMode);
HEL_API void hel_TileDelete(u32 BgNo);
HEL_API void hel_TileShare(u32 TargetBgNo, u32 SourceBgNo, u8 PalNo);
HEL_API void hel_TileReloadGraphic(u32 BgNo, u32 RomTileNo, const u8* pSource);
HEL_API void hel_TilePreloadGraphic(u32 BgNo, u32 RomTileNo);
HEL_API void hel_TileReleaseGraphic(u32 BgNo, u32 RomTileNo);

// Don't use these variables and functions
// directly, use the macros instead!
HEL_API TTileInfo* __hel_TileInfo;
HEL_API TTileSystemInfo* __hel_TileSystemInfo;

// Implemented in helTileFreeMapTile.text.iwram.s
HEL_API void ATTR_FASTFUNC __hel_TileFreeMapTileImplementation(TTileInfo *pTileInfo, u16 *pRamTileNo);

// Implemented in helTileAllocMapTile.text.iwram.s
HEL_API void ATTR_FASTFUNC __hel_TileAllocMapTileImplementation(TTileSystemInfo* pTileSystem, u16 *pDest, const void *pSource);

//////////////////////////////////////////////////////////////////////////
// Text System
//
// Implementation:
//		helBgText.c
//////////////////////////////////////////////////////////////////////////

#define BGTEXT_FLAGS_CREATED				BIT0
#define BGTEXT_FLAGS_DYNAMICTILERELOADING	BIT1
#define BGTEXT_FLAGS_GENERATELUT			BIT2

typedef void (*PPrintCharFunc)(u32 BgNo, u32 X, u32 Y, u32 Character);

typedef struct _TBgTextInfo 
{
	u8 CharW;						// Width in tiles of one character
	u8 CharH;						// Height in tiles of one character
	const void *pMap;				// Pointer to map data
	const void *pData;				// Pointer to lookup table
	u32 Flags;						// Flags
	PPrintCharFunc pPrintCharFunc;	// Pointer to function which is used when printing a character
}TBgTextInfo, *PBgTextInfo;


HEL_API void hel_BgTextInit(void *pBuffer);
HEL_API void hel_BgTextQuit(void);
HEL_API void hel_BgTextCreate(u32 BgNo, u32 CharW, u32 CharH, const void *pMap, const char *pszCharOrder, void *pCharLUT, u32 Flags);
HEL_API void hel_BgTextDelete(u32 BgNo);
HEL_API void hel_BgTextPrint(u32 BgNo, u32 X, u32 Y, u32 Flags, const char *pszString);
HEL_API void hel_BgTextPrintF(u32 BgNo, u32 X, u32 Y, u32 Flags, const char *pszString, ...);

//////////////////////////////////////////////////////////////////////////
// Pad System
//
// Implementation:
//		helPad.c
//////////////////////////////////////////////////////////////////////////

#define PAD_MODE_UP				0
#define PAD_MODE_DOWN			1
#define PAD_MODE_DEFAULT		PAD_MODE_DOWN

#define PAD_BUTTON_A			BIT0
#define PAD_BUTTON_B			BIT1
#define PAD_BUTTON_SELECT		BIT2
#define PAD_BUTTON_START		BIT3
#define PAD_BUTTON_RIGHT		BIT4
#define PAD_BUTTON_LEFT			BIT5
#define PAD_BUTTON_UP			BIT6
#define PAD_BUTTON_DOWN			BIT7
#define PAD_BUTTON_R			BIT8
#define PAD_BUTTON_L			BIT9

#define PAD_BUTTON_ALL			(PAD_BUTTON_A | PAD_BUTTON_B | PAD_BUTTON_SELECT | PAD_BUTTON_START | PAD_BUTTON_RIGHT | \
								PAD_BUTTON_LEFT | PAD_BUTTON_UP | PAD_BUTTON_DOWN | PAD_BUTTON_R | PAD_BUTTON_L)

typedef struct ATTR_PACKED _TPadButtons
{
	union
	{
		struct 
		{
			u8 A:1;      // -A-
			u8 B:1;      // -B-
			u8 Select:1; // -Select-
			u8 Start:1;  // -Start-
			u8 Right:1;  // -Right-
			u8 Left:1;   // -Left-
			u8 Up:1;     // -Up-
			u8 Down:1;   // -Down-
			u8 R:1;      // -R-, ShoulderButton
			u8 L:1;      // -L-, ShoulderButton
		};

		u16 States;
	};

	u8 Data[10*4];	 // Memory to hold key repeat information and such
}TPadButtons, *PPadButtons;

typedef struct _TPad
{
   TPadButtons Pressed; // Holds information about what buttons have been pressed, since last #hel_PadCapture call
   TPadButtons Held;    // Holds information about what buttons are held down, since last #hel_PadCapture call
   u8 Mode;				// Specifies the pad mode, if you want the Pressed-State on button push or release
}TPad, *PPad;


#define __hel_PadQuery()	((TPad*)(&__hel_Pad[0]))

#ifdef HEL_DEBUG
	HEL_API const TPad *hel_PadQuery(void);
#else
	#define hel_PadQuery()	__hel_PadQuery()
#endif


HEL_API void hel_PadInit(void);
HEL_API void hel_PadQuit(void);
HEL_API void hel_PadCapture(void);
HEL_API void hel_PadSetMode(u32 Mode);
HEL_API void hel_PadSetRepeatDelay(u32 Buttons, u8 Value);
HEL_API void hel_PadSetRepeatRate(u32 Buttons, u8 Value);
HEL_API u32 ATTR_WARNUNUSEDRESULT hel_PadGetTicks(u32 Button);

// Do not use this variable, 
// use hel_PadQuery instead!
HEL_API TPad __hel_Pad[];


//////////////////////////////////////////////////////////////////////////
// Palette System
//
// Implementation:
//		helPal.c, helPalBg.c, helPalObj.c, helPalBlend.text.iwram.c
//////////////////////////////////////////////////////////////////////////

#define __hel_PalClear(pPaletteMemory, Count, Color)	hel_DmaSet16((void*)(pPaletteMemory), Color, Count)

#define __hel_PalBgInvert(Index, Count)							hel_PalInvert((void*)MEM_PAL_BG_COL_256(Index), Count)
#define __hel_PalBgInvert16(PaletteBank)                        hel_PalBgInvert((PaletteBank)<<4, 16)
#define __hel_PalBgInvert256()                                  hel_PalBgInvert(0, 256)
#define __hel_PalBgClear16(PaletteBank)                         hel_DmaSet16((void*)MEM_PAL_BG_16(PaletteBank), 0, 16)
#define __hel_PalBgClear256()                                   hel_DmaSet16((void*)MEM_PAL_BG, 0, 256)
#define __hel_PalBgClear(Index, Count, Color)					hel_DmaSet16((void*)(MEM_PAL_BG+((Index)<<1)), Color, Count)
#define __hel_PalBgLoad(SourceData, NumColors)                  hel_DmaCopy16((void*)MEM_PAL_BG, (void*)(SourceData), NumColors)
#define __hel_PalBgLoad16(SourceData, PaletteBank)              hel_DmaCopy16((void*)MEM_PAL_BG_16(PaletteBank), (void*)(SourceData), 16)
#define __hel_PalBgLoad256(SourceData)                          hel_DmaCopy16((void*)MEM_PAL_BG, (void*)(SourceData), 256)
#define __hel_PalBgSave16(pDest, PaletteBank)                   hel_DmaCopy16((void*)(pDest), (void*)MEM_PAL_BG_16(PaletteBank), 16)
#define __hel_PalBgSave256(pDest)                               hel_DmaCopy16((void*)(pDest), (void*)MEM_PAL_BG, 256)
#define __hel_PalBgLoad16UnComp(SourceData, PaletteBank, CompressionType)    hel_SwiUnComp((void*)MEM_PAL_BG_16(PaletteBank), SourceData, CompressionType)
#define __hel_PalBgLoad256UnComp(SourceData, CompressionType)   hel_SwiUnComp((void*)MEM_PAL_BG, SourceData, CompressionType)
#define __hel_PalBgBlend16(pPaletteA, pPaletteB, PaletteBank, Weight) hel_PalBlend((u16*)MEM_PAL_BG_16(PaletteBank), pPaletteA, pPaletteB, Weight, 16)
#define __hel_PalBgBlend256(pPaletteA, pPaletteB, Weight)		hel_PalBlend((u16*)MEM_PAL_BG, pPaletteA, pPaletteB, Weight, 256)

#define __hel_PalObjInvert(Index, Count)						hel_PalInvert((void*)(MEM_PAL_OBJ+((Index)<<1)), Count)
#define __hel_PalObjInvert16(PaletteBank)                       __hel_PalObjInvert((PaletteBank)<<4, 16)
#define __hel_PalObjInvert256()                                 __hel_PalObjInvert(0, 256)
#define __hel_PalObjClear16(PaletteBank)                        hel_DmaSet16((void*)MEM_PAL_OBJ_16(PaletteBank), 0, 16)
#define __hel_PalObjClear256()                                  hel_DmaSet16((void*)MEM_PAL_OBJ, 0, 256)
#define __hel_PalObjClear(Index, Count, Color)					hel_DmaSet16((void*)(MEM_PAL_OBJ+((Index)<<1)), Color, Count)
#define __hel_PalObjLoad(SourceData, NumColors)                 hel_DmaCopy16((void*)MEM_PAL_OBJ, (void*)(SourceData), NumColors)
#define __hel_PalObjLoad16(SourceData, PaletteBank)             hel_DmaCopy16((void*)MEM_PAL_OBJ_16(PaletteBank), (void*)(SourceData), 16)
#define __hel_PalObjLoad256(SourceData)                         hel_DmaCopy16((void*)MEM_PAL_OBJ, (void*)(SourceData), 256)
#define __hel_PalObjSave16(pDest, PaletteBank)                  hel_DmaCopy16((void*)(pDest), (void*)MEM_PAL_OBJ_16(PaletteBank), 16)
#define __hel_PalObjSave256(pDest)                              hel_DmaCopy16((void*)(pDest), (void*)MEM_PAL_OBJ, 256)
#define __hel_PalObjLoad256UnComp(SourceData, CompressionType)  hel_SwiUnComp((void*)MEM_PAL_OBJ, SourceData, CompressionType)
#define __hel_PalObjLoad16UnComp(SourceData, PaletteBank, CompressionType)   hel_SwiUnComp((void*)MEM_PAL_OBJ_16(PaletteBank), SourceData, CompressionType)
#define __hel_PalObjBlend16(pPaletteA, pPaletteB, PaletteBank, Weight)	hel_PalBlend((u16*)MEM_PAL_OBJ_16(PaletteBank), pPaletteA, pPaletteB, Weight, 16)
#define __hel_PalObjBlend256(pPaletteA, pPaletteB, Weight)		hel_PalBlend((u16*)MEM_PAL_OBJ, pPaletteA, pPaletteB, Weight, 256)

#ifdef HEL_DEBUG
	// Palette Functions
	HEL_API void hel_PalClear(void *pPaletteMemory, u32 Count, u32 Color);

	// Background Palette Functions
	HEL_API void hel_PalBgInvert(u32 Index, u32 Count);
	HEL_API void hel_PalBgInvert16(u32 PaletteBank);
	HEL_API void hel_PalBgInvert256(void);
	HEL_API void hel_PalBgClear(u32 Index, u32 Count, u32 Color);
	HEL_API void hel_PalBgClear16(u32 PaletteBank);
	HEL_API void hel_PalBgClear256(void);
	HEL_API void hel_PalBgLoad(const u16 *SourceData, u32 NumColors);
	HEL_API void hel_PalBgLoad16(const u16 *SourceData, u32 PaletteBank);
	HEL_API void hel_PalBgLoad256(const u16 *SourceData);
	HEL_API void hel_PalBgSave16(void *pDest, u32 PaletteBank);
	HEL_API void hel_PalBgSave256(void *pDest);
	HEL_API void hel_PalBgLoad16UnComp(const u16 *SourceData, u32 PaletteBank, u32 CompressionType);
	HEL_API void hel_PalBgLoad256UnComp(const u16 *SourceData, u32 CompressionType);
	HEL_API void hel_PalBgBlend16(const u16 *pPaletteA, const u16 *pPaletteB, u32 PaletteBank, u32 Weight);
	HEL_API void hel_PalBgBlend256(const u16 *pPaletteA, const u16 *pPaletteB, u32 Weight);

	// Object Palette Functions
	HEL_API void hel_PalObjInvert(u32 Index, u32 Count);
	HEL_API void hel_PalObjInvert16(u32 PaletteBank);
	HEL_API void hel_PalObjInvert256(void);
	HEL_API void hel_PalObjClear(u32 Index, u32 Count, u32 Color);
	HEL_API void hel_PalObjClear16(u32 PaletteBank);
	HEL_API void hel_PalObjClear256(void);
	HEL_API void hel_PalObjLoad(const u16 *SourceData, u32 NumColors);
	HEL_API void hel_PalObjLoad16(const u16 *SourceData, u32 PaletteBank);
	HEL_API void hel_PalObjLoad256(const u16 *SourceData);
	HEL_API void hel_PalObjSave16(void *pDest, u32 PaletteBank);
	HEL_API void hel_PalObjSave256(void *pDest);
	HEL_API void hel_PalObjLoad16UnComp(const u16 *SourceData, u32 PaletteBank, u32 CompressionType);
	HEL_API void hel_PalObjLoad256UnComp(const u16 *SourceData, u32 CompressionType);
	HEL_API void hel_PalObjBlend16(const u16 *pPaletteA, const u16 *pPaletteB, u32 PaletteBank, u32 Weight);
	HEL_API void hel_PalObjBlend256(const u16 *pPaletteA, const u16 *pPaletteB, u32 Weight);
#else
	// Palette Functions
	#define hel_PalClear(pPaletteMemory, Count, Color)			__hel_PalClear(pPaletteMemory, Count, Color)

	// Background Palette Functions
	#define hel_PalBgInvert(Index, Count)						__hel_PalBgInvert(Index, Count)	
	#define hel_PalBgInvert16(PaletteBank)                      __hel_PalBgInvert16(PaletteBank)
	#define hel_PalBgInvert256()                                __hel_PalBgInvert256()
	#define hel_PalBgClear(Index, Count, Color)					__hel_PalBgClear(Index, Count, Color)
	#define hel_PalBgClear16(PaletteBank)                       __hel_PalBgClear16(PaletteBank)
	#define hel_PalBgClear256()                                 __hel_PalBgClear256()
	#define hel_PalBgLoad(SourceData, NumColors)                __hel_PalBgLoad(SourceData, NumColors)
	#define hel_PalBgLoad16(SourceData, PaletteBank)            __hel_PalBgLoad16(SourceData, PaletteBank)
	#define hel_PalBgLoad256(SourceData)                        __hel_PalBgLoad256(SourceData)
	#define hel_PalBgSave16(pDest, PaletteBank)                 __hel_PalBgSave16(pDest, PaletteBank)  
	#define hel_PalBgSave256(pDest)                             __hel_PalBgSave256(pDest)
	#define hel_PalBgLoad16UnComp(SourceData, PaletteBank, CompressionType)	__hel_PalBgLoad16UnComp(SourceData, PaletteBank, CompressionType)
	#define hel_PalBgLoad256UnComp(SourceData, CompressionType)	__hel_PalBgLoad256UnComp(SourceData, CompressionType)
	#define hel_PalBgBlend16(pPaletteA, pPaletteB, PaletteBank, Weight)	__hel_PalBgBlend16(pPaletteA, pPaletteB, PaletteBank, Weight)
	#define hel_PalBgBlend256(pPaletteA, pPaletteB, Weight)		__hel_PalBgBlend256(pPaletteA, pPaletteB, Weight)

	// Object Palette Functions
	#define hel_PalObjInvert(Index, Count)						__hel_PalObjInvert(Index, Count)	
	#define hel_PalObjInvert16(PaletteBank)                     __hel_PalObjInvert16(PaletteBank)
	#define hel_PalObjInvert256()                               __hel_PalObjInvert256()
	#define hel_PalObjClear(Index, Count, Color)				__hel_PalObjClear(Index, Count, Color)
	#define hel_PalObjClear16(PaletteBank)                      __hel_PalObjClear16(PaletteBank)
	#define hel_PalObjClear256()                                __hel_PalObjClear256()
	#define hel_PalObjLoad(SourceData, NumColors)               __hel_PalObjLoad(SourceData, NumColors)
	#define hel_PalObjLoad16(SourceData, PaletteBank)           __hel_PalObjLoad16(SourceData, PaletteBank)
	#define hel_PalObjLoad256(SourceData)                       __hel_PalObjLoad256(SourceData)
	#define hel_PalObjSave16(pDest, PaletteBank)                __hel_PalObjSave16(pDest, PaletteBank)
	#define hel_PalObjSave256(pDest)                            __hel_PalObjSave256(pDest)
	#define hel_PalObjLoad16UnComp(SourceData, PaletteBank, CompressionType)   __hel_PalObjLoad16UnComp(SourceData, PaletteBank, CompressionType)
	#define hel_PalObjLoad256UnComp(SourceData, CompressionType)  __hel_PalObjLoad256UnComp(SourceData, CompressionType)
	#define hel_PalObjBlend16(pPaletteA, pPaletteB, PaletteBank, Weight)	__hel_PalObjBlend16(pPaletteA, pPaletteB, PaletteBank, Weight)
	#define hel_PalObjBlend256(pPaletteA, pPaletteB, Weight)	__hel_PalObjBlend256(pPaletteA, pPaletteB, Weight)
#endif


HEL_API void hel_PalInvert(void *pPaletteMemory, u32 Count);
HEL_API void hel_PalBlend(u16 *pTarget, const u16 *pPaletteA, const u16 *pPaletteB, u32 Step, u32 Count);
HEL_API void hel_PalFade(void *pPaletteMemory, const void *pSource, u32 Count, u32 Color, u32 Step);



//////////////////////////////////////////////////////////////////////////
// Object System
//////////////////////////////////////////////////////////////////////////

#define OBJ_ERROR_VRAMFULL			200
#define OBJ_ERROR_ALLSLOTSTAKEN		201

#define OBJ_SHAPE_SQUARE			0
#define OBJ_SHAPE_HORIZONTAL		1
#define OBJ_SHAPE_VERTICAL			2

#define OBJ_RESTORE_UPDATEGFX		BIT0

// Bit 0..7 is used for allocated
// block amount in Vram. One block
// equals 32 bytes. Up to 128 blocks
// can be allocated by one object.
#define OBJ_FLAGS_BLOCKSUSED_MASK	(~(BIT0 | BIT1 | BIT2 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7))
#define OBJ_FLAGS_BLOCKSUSED_SHIFT	0

#define OBJ_FLAGS_CREATED			BIT(OBJ_FLAGS_CREATED_SHIFT)
#define OBJ_FLAGS_CREATED_MASK		(~OBJ_FLAGS_CREATED)
#define OBJ_FLAGS_CREATED_SHIFT		8

#define OBJ_FLAGS_VISIBLE			BIT(OBJ_FLAGS_VISIBLE_SHIFT)
#define OBJ_FLAGS_VISIBLE_MASK		(~OBJ_FLAGS_VISIBLE)
#define OBJ_FLAGS_VISIBLE_SHIFT		9

#define OBJ_FLAGS_UPDATEGFX			BIT(OBJ_FLAGS_UPDATEGFX_SHIFT)
#define OBJ_FLAGS_UPDATEGFX_MASK	(~OBJ_FLAGS_UPDATEGFX)
#define OBJ_FLAGS_UPDATEGFX_SHIFT	10

#define OBJ_FLAGS_CLONE				BIT(OBJ_FLAGS_CLONE_SHIFT)
#define OBJ_FLAGS_CLONE_MASK		(~OBJ_FLAGS_CLONE)
#define OBJ_FLAGS_CLONE_SHIFT		11

#define OBJ_FLAGS_OAMSPOT_MASK		(~(BIT(16) | BIT(17) | BIT(18) | BIT(19) | BIT(20) | BIT(21) | BIT(22) | BIT(23)))
#define OBJ_FLAGS_OAMSPOT_SHIFT		16



/*!
\ingroup helObj
\brief  Compose object attribute 0 value
\hideinitializer

	The #OBJ_ATTR0 helper macro can be used to compose the attribute 0 value, which
	you can use to feed #hel_ObjCreateEx with. For details please refer to \ref SectionGlossaryG_GBATek.

\param[in]
	ObjShape	Specifies the object shape, which in conjunction with the \c ObjSize parameter
						of #OBJ_ATTR1, represents the object dimension. This argument can be one of
						the following values (see notes):
							- \c OBJ_SHAPE_SQUARE
							- \c OBJ_SHAPE_HORIZONTAL
							- \c OBJ_SHAPE_VERTICAL

\param[in]
	ObjMode		Specifies the object mode. This can be one of these values:
							- \c OBJ_MODE_NORMAL
							- \c OBJ_MODE_SEMITRANSPARENT
							- \c OBJ_MODE_OBJWINDOW

\param[in]
	ColMode		Specifies the object color mode (16 or 256 colors). Can be on of these
						values:
							- \c COLORS_16
							- \c COLORS_256

\param[in]
	Mosaic		Enables or disables mosaic processing for the
						object in question. Set it to \c TRUE or \c FALSE.

\param[in]
	DblSize		Enables or disables double-size for the
						object in question. Set it to \c TRUE or \c FALSE. Objects which
						can rotate usually require the double-size attribute bit set.

\param[in]
	Y					Vertical screen coordinate in pixels

\note
Here is a listing what size/shape pairs form which object dimension: 
\verbatim
Size  Square   Horizontal  Vertical
0     8x8      16x8        8x16
1     16x16    32x8        8x32
2     32x32    32x16       16x32
3     64x64    64x32       32x64
\endverbatim

\return
	Returns the composed attribute 0 value (fits in a 16bit word).

\sa
	#OBJ_ATTR1, #OBJ_ATTR2, #hel_ObjCreateEx
*/
#define OBJ_ATTR0(ObjShape, ObjMode, ColMode, Mosaic, DblSize, Y)\
	((ObjShape)<<14) | ((ColMode)<<13) | ((Mosaic)<<12) | ((ObjMode)<<10) | ((DblSize)<<9) | ((Y) & 0x00FF)


/*!
\ingroup helObj
\brief  Compose object attribute 1 value
\hideinitializer

	The #OBJ_ATTR1 helper macro can be used to compose the attribute 1 value, which
	you can use to feed #hel_ObjCreateEx with. For details please refer to \ref SectionGlossaryG_GBATek.

\param[in]
	ObjSize		Specifies the object size, which in conjunction with the \c ObjShape parameter
						of #OBJ_ATTR0, represents the object dimension. This argument must between
						0 and 3. See notes.


\param[in]
	VFlip			The \c VFlip argument can be used to enable or disable vertical flipping.
						Set it to \c TRUE to enable, or \c FALSE to disable v-flip.

\param[in]
	HFlip			The \c VFlip argument can be used to enable or disable horizontal flipping.
						Set it to \c TRUE to enable, or \c FALSE to disable h-flip.

\param[in]
	X					Horizontal screen coordinate in pixels

\note
Here is a listing what size/shape pairs form which object dimension: 
\verbatim
Size  Square   Horizontal  Vertical
0     8x8      16x8        8x16
1     16x16    32x8        8x32
2     32x32    32x16       16x32
3     64x64    64x32       32x64
\endverbatim


\return
	Returns the composed attribute 1 value (fits in a 16bit word).

\sa
	#OBJ_ATTR0, #OBJ_ATTR2, #hel_ObjCreateEx
*/
#define OBJ_ATTR1(ObjSize, VFlip, HFlip, X)\
	((ObjSize)<<14) | ((VFlip)<<13) | ((HFlip)<<12) | ((X) & 0x01FF)


/*!
\ingroup helObj
\brief  Compose object attribute 2 value
\hideinitializer

	The #OBJ_ATTR2 helper macro can be used to compose the attribute \c 2 value, which
	you can use to feed #hel_ObjCreateEx with. For details please refer to \ref SectionGlossaryG_GBATek.

\param[in]
	PalNo			Palettebank/number which the object use. This value is of interest
						for objects with 16 colors only. Must between \c 0..15, set it to \c 0
						for 256 color objects.

\param[in]
	Prio			Priority relative to backgrounds. Must between \c 0..3. \c 0 equals highest
						priority which means before top background.


\return
	Returns the composed attribute 2 value (fits in a 16bit word).

\sa
	#OBJ_ATTR0, #OBJ_ATTR1, #hel_ObjCreateEx
*/
#define OBJ_ATTR2(PalNo, Prio)\
	((PalNo)<<12) | ((Prio)<<10)


typedef struct _TObjInfo 
{
	u16 Attr[3];		// Shadowed OAM attributes
	THandle Next;		// Handle to next object in list
	THandle Prev;		// Handle to previous object in list
	u32 Flags;			// Flags that control object behavior and provide additional information
	const void* pSrc;	// Pointer to source graphic data
} TObjInfo, *PObjInfo;
 
#define __hel_ObjExists(Handle)             (HANDLE_ISVALID(Handle)?((__hel_ObjInfo[Handle].Flags >> OBJ_FLAGS_CREATED_SHIFT) & 1):FALSE)
#define __hel_ObjIsClone(Handle)			((__hel_ObjInfo[Handle].Flags >> OBJ_FLAGS_CLONE_SHIFT) & 1)
#define __hel_ObjIsVisible(Handle)          ((__hel_ObjInfo[Handle].Flags >> OBJ_FLAGS_VISIBLE_SHIFT) & 1)
#define __hel_ObjIsHFlip(Handle)            ((__hel_ObjInfo[Handle].Attr[1] >> 12) & 1)
#define __hel_ObjIsVFlip(Handle)            ((__hel_ObjInfo[Handle].Attr[1] >> 13) & 1)
#define __hel_ObjIsMosaic(Handle)           ((__hel_ObjInfo[Handle].Attr[0] >> 12) & 1)
#define __hel_ObjIsRotScale(Handle)         ((__hel_ObjInfo[Handle].Attr[0] >>  8) & 1)
#define __hel_ObjGetColorMode(Handle)       ((__hel_ObjInfo[Handle].Attr[0] >> 13) & 1)
#define __hel_ObjGetMode(Handle)            ((__hel_ObjInfo[Handle].Attr[0] >> 10) & 2)
#define __hel_ObjBringToFront(Handle)       do { if(__hel_ObjFirst != (Handle)) hel_ObjSetBefore(Handle, __hel_ObjFirst); } while(0)
#define __hel_ObjSendToBack(Handle)			do { if(__hel_ObjLast != (Handle)) hel_ObjSetBehind(Handle, __hel_ObjLast); } while(0)
#define __hel_ObjGetFirst()					__hel_ObjFirst
#define __hel_ObjGetLast()					__hel_ObjLast
#define __hel_ObjGetNext(Handle)			__hel_ObjInfo[Handle].Next
#define __hel_ObjGetPrev(Handle)			__hel_ObjInfo[Handle].Prev
#define __hel_ObjHideAll()					hel_ObjSetVisibleAll(FALSE)
#define __hel_ObjShowAll()					hel_ObjSetVisibleAll(TRUE)
#define __hel_ObjSetVisible(Handle, Value)	do { u32 *pFlags = &__hel_ObjInfo[Handle].Flags; *pFlags &= ~OBJ_FLAGS_VISIBLE; *pFlags |= ((Value)<<OBJ_FLAGS_VISIBLE_SHIFT); } while(0)
#define __hel_ObjSetX(Handle, X)            do { u16 *pAttr = &__hel_ObjInfo[Handle].Attr[1]; *pAttr &= 0xFE00; *pAttr |= ((X) & 0x01FF); } while(0)
#define __hel_ObjSetY(Handle, Y)            do { u16 *pAttr = &__hel_ObjInfo[Handle].Attr[0]; *pAttr &= 0xFF00; *pAttr |= ((Y) & 0x00FF); } while(0)
#define __hel_ObjSetXY(Handle, X, Y)        do { u32 *pAttr = (u32*)&__hel_ObjInfo[Handle]; *pAttr &= 0xfe00ff00; *pAttr |= (((X)&511) << 16) | (((Y)&255)); } while(0)
#define __hel_ObjSetHFlip(Handle, Value)    do { u16 *pAttr = &__hel_ObjInfo[Handle].Attr[1]; *pAttr &= 0xEFFF; *pAttr |= ((Value) << 12); } while(0)
#define __hel_ObjSetVFlip(Handle, Value)    do { u16 *pAttr = &__hel_ObjInfo[Handle].Attr[1]; *pAttr &= 0xDFFF; *pAttr |= ((Value) << 13); } while(0)
#define __hel_ObjSetHVFlip(Handle, HFlip, VFlip)     do { u16 *pAttr = &__hel_ObjInfo[Handle].Attr[1]; *pAttr &= ~(BIT12|BIT13); *pAttr |= (((HFlip) << 12) | ((VFlip) << 13)); } while(0)
#define __hel_ObjSetPrio(Handle, Prio)				 do { u16 *pAttr = &__hel_ObjInfo[Handle].Attr[2]; *pAttr &= 0xF3FF; *pAttr |= ((Prio) << 10); } while(0)
#define __hel_ObjGetGfxSlot(Handle)         (__hel_ObjInfo[Handle].Attr[2] & 0x03ff)
#define __hel_ObjSetMode(Handle, Mode)         do { u16 *pAttr = &__hel_ObjInfo[Handle].Attr[0]; *pAttr &= 0xf3ff; *pAttr |= ((Mode) << 10); } while(0)
#define __hel_ObjGetPrio(Handle)               ((__hel_ObjInfo[Handle].Attr[2] >> 10) & 3)
#define __hel_ObjSelectRotScale(Handle, RotNo) do { u16 *pAttr = &__hel_ObjInfo[Handle].Attr[1]; *pAttr &= 0xC1FF; *pAttr |= ((RotNo)<<9); } while(0)
#define __hel_ObjGetShape(Handle)              (__hel_ObjInfo[Handle].Attr[0]>>14)
#define __hel_ObjGetSize(Handle)               (__hel_ObjInfo[Handle].Attr[1]>>14)
#define __hel_ObjGetSizeU16(Handle)            (((__hel_ObjInfo[Handle].Flags & (~OBJ_FLAGS_BLOCKSUSED_MASK))<<5)>>1)
#define __hel_ObjUpdateGfx(Handle, _pSrc)				do { __hel_ObjInfo[Handle].pSrc = (const void*)(_pSrc); __hel_ObjInfo[Handle].Flags |= OBJ_FLAGS_UPDATEGFX; } while(0)
#define __hel_ObjUpdateGfxUnComp(Handle, _pSrc, CompressionType)	__hel_SwiUnComp((void*)MEM_OBJ_SLOT_PTR(__hel_ObjGetGfxSlot(Handle)), (void*)(_pSrc), CompressionType)
#define __hel_ObjGetFreeSlots()									__hel_ObjCount
#define __hel_ObjGetAttribute(Handle, AttributeNumber)	__hel_ObjInfo[Handle].Attr[AttributeNumber]
#define __hel_ObjSetPalette(Handle, PaletteBank)	do { u16 *pAttr = &__hel_ObjInfo[Handle].Attr[2]; *pAttr &= 0xFFF; *pAttr |= ((PaletteBank) << 12); } while(0)
#define __hel_ObjGetPalette(Handle)							(__hel_ObjInfo[Handle].Attr[2] >> 12)
#define __hel_ObjSetMosaic(Handle, Enable)			do { u16 *pAttr = &__hel_ObjInfo[Handle].Attr[0]; *pAttr &= 0xEFFF; *pAttr |= ((Enable) << 12); } while(0)
#define __hel_ObjGetX(Handle)					(__hel_ObjInfo[Handle].Attr[1] & 511)
#define __hel_ObjGetY(Handle)					(__hel_ObjInfo[Handle].Attr[0] & 255)
#define __hel_ObjGetWidth(Handle)				HEL_OBJ_SHAPESIZE_TO_WIDTH[__hel_ObjGetSize(Handle)][__hel_ObjGetShape(Handle)]
#define __hel_ObjGetHeight(Handle)				HEL_OBJ_SHAPESIZE_TO_HEIGHT[__hel_ObjGetSize(Handle)][__hel_ObjGetShape(Handle)]
#define __hel_ObjGetOamSlot(Handle)				((__hel_ObjInfo[Handle].Flags & ~OBJ_FLAGS_OAMSPOT_MASK) >> OBJ_FLAGS_OAMSPOT_SHIFT)

#ifdef HEL_DEBUG
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjExists(THandle Handle);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjIsClone(THandle Handle);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjIsHFlip(THandle Handle);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjIsVFlip(THandle Handle);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjIsVisible(THandle Handle);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjIsMosaic(THandle Handle);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjIsRotScale(THandle Handle);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjGetColorMode(THandle Handle);
	HEL_API void hel_ObjBringToFront(THandle Handle);
	HEL_API void hel_ObjSendToBack(THandle Handle);
	HEL_API THandle ATTR_WARNUNUSEDRESULT hel_ObjGetFirst(void);
	HEL_API THandle ATTR_WARNUNUSEDRESULT hel_ObjGetLast(void);
	HEL_API THandle ATTR_WARNUNUSEDRESULT hel_ObjGetNext(THandle Handle);
	HEL_API THandle ATTR_WARNUNUSEDRESULT hel_ObjGetPrev(THandle Handle);
	HEL_API void hel_ObjHideAll(void);
	HEL_API void hel_ObjShowAll(void);
	HEL_API void hel_ObjSetVisible(THandle Handle, u32 Value);
	HEL_API void hel_ObjSetX(THandle Handle, s16 X);
	HEL_API void hel_ObjSetY(THandle Handle, s16 X);
	HEL_API void hel_ObjSetXY(THandle Handle, s16 X, s16 Y);
	HEL_API void hel_ObjSetHFlip(THandle Handle, u32 Value);
	HEL_API void hel_ObjSetVFlip(THandle Handle, u32 Value);
	HEL_API void hel_ObjSetHVFlip(THandle Handle, u32 HFlip, u32 VFlip);
	HEL_API void hel_ObjSetPrio(THandle Handle, u32 Prio);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjGetGfxSlot(THandle Handle);
	HEL_API void hel_ObjSetMode(THandle Handle, u32 Mode);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjGetPrio(THandle Handle);
	HEL_API void hel_ObjSelectRotScale(THandle Handle, THandle RotNo);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjGetShape(THandle Handle);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjGetSize(THandle Handle);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjGetSizeU16(THandle Handle);
	HEL_API void hel_ObjUpdateGfx(THandle Handle, const void* pSrc);
	HEL_API void hel_ObjUpdateGfxUnComp(THandle Handle, const void* pSrc, u32 CompressionType);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjGetFreeSlots(void);
	HEL_API u16 ATTR_WARNUNUSEDRESULT hel_ObjGetAttribute(THandle Handle, u32 AttributeNumber);
	HEL_API void hel_ObjSetPalette(THandle Handle, u32 PaletteBank);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjGetPalette(THandle Handle);
	HEL_API void hel_ObjSetMosaic(THandle Handle, u32 Enable);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjGetX(THandle Handle);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjGetY(THandle Handle);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjGetWidth(THandle Handle);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjGetHeight(THandle Handle);
	HEL_API u32 ATTR_WARNUNUSEDRESULT hel_ObjGetOamSlot(THandle Handle);
#else
	#define hel_ObjExists(Handle)                __hel_ObjExists(Handle)
	#define hel_ObjIsClone(Handle)               __hel_ObjIsClone(Handle)
	#define hel_ObjIsHFlip(Handle)               __hel_ObjIsHFlip(Handle)
	#define hel_ObjIsVFlip(Handle)               __hel_ObjIsVFlip(Handle)
	#define hel_ObjIsVisible(Handle)             __hel_ObjIsVisible(Handle)
	#define hel_ObjIsMosaic(Handle)              __hel_ObjIsMosaic(Handle)
	#define hel_ObjIsRotScale(Handle)            __hel_ObjIsRotScale(Handle)
	#define hel_ObjGetColorMode(Handle)          __hel_ObjGetColorMode(Handle)
	#define hel_ObjGetMode(Handle)               __hel_ObjGetMode(Handle)
	#define hel_ObjBringToFront(Handle)          __hel_ObjBringToFront(Handle)
	#define hel_ObjSendToBack(Handle)			__hel_ObjSendToBack(Handle)
	#define hel_ObjGetFirst()					__hel_ObjGetFirst()
	#define hel_ObjGetLast()					__hel_ObjGetLast()
	#define hel_ObjGetNext(Handle)				__hel_ObjGetNext(Handle)
	#define hel_ObjGetPrev(Handle)				__hel_ObjGetPrev(Handle)
	#define hel_ObjHideAll()                     __hel_ObjHideAll()
	#define hel_ObjShowAll()                     __hel_ObjShowAll()
	#define hel_ObjSetVisible(Handle, Value)     __hel_ObjSetVisible(Handle, Value)
	#define hel_ObjSetX(Handle, X)               __hel_ObjSetX(Handle, X)
	#define hel_ObjSetY(Handle, Y)               __hel_ObjSetY(Handle, Y)
	#define hel_ObjSetXY(Handle, X, Y)           __hel_ObjSetXY(Handle, X, Y)
	#define hel_ObjSetHFlip(Handle, Value)       __hel_ObjSetHFlip(Handle, Value)
	#define hel_ObjSetVFlip(Handle, Value)       __hel_ObjSetVFlip(Handle, Value)
	#define hel_ObjSetHVFlip(Handle, HFlip, VFlip)  __hel_ObjSetHVFlip(Handle, HFlip, VFlip)
	#define hel_ObjSetPrio(Handle, Prio)         __hel_ObjSetPrio(Handle, Prio)
	#define hel_ObjGetGfxSlot(Handle)            __hel_ObjGetGfxSlot(Handle)
	#define hel_ObjSetMode(Handle, Mode)         __hel_ObjSetMode(Handle, Mode)
	#define hel_ObjGetPrio(Handle)               __hel_ObjGetPrio(Handle)
	#define hel_ObjSelectRotScale(Handle, RotNo) __hel_ObjSelectRotScale(Handle, RotNo)
	#define hel_ObjGetShape(Handle)             __hel_ObjGetShape(Handle)
	#define hel_ObjGetSize(Handle)              __hel_ObjGetSize(Handle)
	#define hel_ObjGetSizeU16(Handle)           __hel_ObjGetSizeU16(Handle)
	#define hel_ObjUpdateGfx(Handle, pSrc)      __hel_ObjUpdateGfx(Handle, pSrc)
	#define hel_ObjUpdateGfxUnComp(Handle, pSrc, CompressionType)    __hel_ObjUpdateGfxUnComp(Handle, pSrc, CompressionType)
	#define hel_ObjGetFreeSlots()				__hel_ObjGetFreeSlots()
	#define hel_ObjGetAttribute(Handle, AttributeNumber)	__hel_ObjGetAttribute(Handle, AttributeNumber)
	#define hel_ObjSetPalette(Handle, PaletteBank)	__hel_ObjSetPalette(Handle, PaletteBank)
	#define hel_ObjGetPalette(Handle)			__hel_ObjGetPalette(Handle)
	#define hel_ObjSetMosaic(Handle, Enable)	__hel_ObjSetMosaic(Handle, Enable)
	#define hel_ObjGetX(Handle)					__hel_ObjGetX(Handle)
	#define hel_ObjGetY(Handle)					__hel_ObjGetY(Handle)	
	#define hel_ObjGetWidth(Handle)				__hel_ObjGetWidth(Handle)
	#define hel_ObjGetHeight(Handle)			__hel_ObjGetHeight(Handle)
	#define hel_ObjGetOamSlot(Handle)			__hel_ObjGetOamSlot(Handle)
#endif  // HEL_DEBUG

/*!
\ingroup helObj
\brief  Create an object
\hideinitializer

	The #hel_ObjCreate function creates a new object and returns
	a handle to the created resource.

\param[in]
	pSrc			Pointer to source graphic. This parameter can be \p NULL. Please
						see #hel_ObjCreateEx for more information.
\param[in]
	ObjShape	Shape of object. This can be one of the following values:
						- \p OBJ_SHAPE_SQUARE
						- \p OBJ_SHAPE_VERTICAL
						- \p OBJ_SHAPE_HORIZONTAL
\param[in]
	ObjSize		Size of object. The size is related to \p ObjShape.
\verbatim
Size  Square   Horizontal  Vertical
0     8x8      16x8        8x16
1     16x16    32x8        8x32
2     32x32    32x16       16x32
3     64x64    64x32       32x64
\endverbatim
\param[in]
	ObjMode		Mode of object, can be one of the following values:
						- \p OBJ_MODE_NORMAL
						- \p OBJ_MODE_OBJWINDOW
						- \p OBJ_MODE_SEMITRANSPARENT
\param[in]
	ColMode		Color mode of object. Can be one of these values:
						- \p COLORS_16
						- \p COLORS_256
\param[in]
	PalNo			Palette number. If the object is using a 16 color palette (\p ColMode equals 0),
						then must \p PalNo contain the palette number of the 16 color palette.
\param[in]
	Mosaic		Specifies if the object should activate mosaic processing. Set to \p FALSE to disable,
						or to \p TRUE to enable mosaic processing.
\param[in]
	HFlip			Specifies if the object should be horizontally flipped. Set to \p TRUE to
						enable horizontal flipping or \p FALSE to disable.
\param[in]
	VFlip			Specifies if the object should be vertically flipped. Set to \p TRUE to
						enable vertical flipping or \p FALSE to disable.
\param[in]
	Prio			Priority relative to backgrounds. Can between 0..3, 0 equals highest
						priority.
\param[in]
	DblSize		Specifies if the object should be double-sized. Can be \p TRUE or \p FALSE.
\param[in]
	X					Horizontal screen coordinate specified in pixels
\param[in]
	Y					Vertical screen coordinate specified in pixels

\return
	The #hel_ObjCreate function will return a handle to the created object on success, otherwise
	one of the following error code:
	\li \c OBJ_ERROR_VRAMFULL      : Vram is full. Required memory in Vram could not be allocated.
	\li \c OBJ_ERROR_ALLSLOTSTAKEN : All 128 object slots are already in use.

\sa
	#hel_ObjCreate256, #hel_ObjCreate16
*/
#define hel_ObjCreate(pSrc, ObjShape, ObjSize, ObjMode, ColMode, PalNo, Mosaic, HFlip, VFlip, Prio, DblSize, X, Y) \
  hel_ObjCreateEx\
    (\
      (void*)(pSrc),\
      OBJ_ATTR0(ObjShape, ObjMode, ColMode, Mosaic, DblSize, Y),\
      OBJ_ATTR1(ObjSize, VFlip, HFlip, X),\
      OBJ_ATTR2(PalNo, Prio)\
    )

/*!
\ingroup helObj
\brief  Create a 256 color object
\hideinitializer

	Please see #hel_ObjCreate for details.
*/
#define hel_ObjCreate256(pSrc, ObjShape, ObjSize, ObjMode, X, Y) \
  hel_ObjCreate\
    (             \
      pSrc,       \
      ObjShape,   \
      ObjSize,    \
      ObjMode,    \
      COLORS_256,	\
      0,          \
      FALSE,      \
      FALSE,      \
      FALSE,      \
      0,          \
      FALSE,      \
      X,          \
      Y           \
    )

/*!
\ingroup helObj
\brief  Create a 16 color object
\hideinitializer

	Please see #hel_ObjCreate for details.
*/
#define hel_ObjCreate16(pSrc, ObjShape, ObjSize, ObjMode, PalNo, X, Y) \
  hel_ObjCreate\
    (             \
      pSrc,       \
      ObjShape,   \
      ObjSize,    \
      ObjMode,    \
      COLORS_16,	\
      PalNo,      \
      FALSE,      \
      FALSE,      \
      FALSE,      \
      0,          \
      FALSE,      \
      X,          \
      Y           \
    )


// helObj.c
HEL_API void hel_ObjInit(void *pBuffer);
HEL_API void hel_ObjQuit(void);
HEL_API void hel_ObjSetVisibleAll(u32 Show);
HEL_API HEL_INLINE void hel_ObjSetMosaicSize(u32 HSize, u32 VSize);
HEL_API void hel_ObjSetRotScale(THandle Handle, u32 Value);
HEL_API void hel_ObjSetDblSize(THandle Handle, u32 Value);
HEL_API void hel_ObjSetBefore(THandle Handle, THandle TargetHandle);
HEL_API void hel_ObjSetBehind(THandle Handle, THandle TargetHandle);
HEL_API void hel_ObjBackup(void *pDest, u32 Flags);
HEL_API void hel_ObjRestore(const void *pSource, u32 Flags);

// helObjTransmit.text.iwram.s
HEL_API void hel_ObjTransmit(void);

// helObjClearOAM.s
HEL_API void hel_ObjClearOAM(void);

// helObj.text.iwram.c
HEL_API THandle ATTR_FASTFUNC ATTR_WARNUNUSEDRESULT hel_ObjCreateEx(const void *pSrc, u32 Attr0, u32 Attr1, u32 Attr2);
HEL_API THandle ATTR_FASTFUNC ATTR_WARNUNUSEDRESULT hel_ObjClone(THandle Handle, s32 X, s32 Y);
HEL_API void ATTR_FASTFUNC hel_ObjDelete(THandle Handle);

// Do not use these variables.
// Use the above macros instead!
HEL_API u32 __hel_ObjFirst;
HEL_API u32 __hel_ObjLast;
HEL_API u32 __hel_ObjCount;
HEL_API TObjInfo *__hel_ObjInfo;

// Do not use these constants,
// Use the above macros instead!
// hel_ObjGetWidth, hel_ObjGetHeight
HEL_API const u32 HEL_OBJ_SHAPESIZE_TO_WIDTH[4][4];
HEL_API const u32 HEL_OBJ_SHAPESIZE_TO_HEIGHT[4][4];
HEL_API const u32 HEL_OBJ_SHAPESIZE_TO_HALFWORD[2][4][4];


//////////////////////////////////////////////////////////////////////////
// Object Rotation/Scaling System
//
// Implementation:
//		helObjRotScale.c
//////////////////////////////////////////////////////////////////////////

typedef struct ATTR_PACKED _TObjRotScaleInfo 
{
	s16 ScaleX;
	s16 ScaleY;
	u16 Angle;
} ATTR_PACKED TObjRotScaleInfo, *PObjRotScaleInfo;

HEL_API void hel_ObjRotScaleInit(void *pBuffer);
HEL_API void hel_ObjRotScaleQuit(void);
HEL_API void hel_ObjRotScaleBackup(void *pDest, u32 Flags);
HEL_API void hel_ObjRotScaleRestore(const void *pSource, u32 Flags);
HEL_API THandle ATTR_WARNUNUSEDRESULT hel_ObjRotScaleCreate(u32 Angle, s32 ScaleX, s32 ScaleY);
HEL_API THandle ATTR_WARNUNUSEDRESULT hel_ObjRotScaleClone(THandle Handle);
HEL_API void hel_ObjRotScaleDelete(THandle Handle);
HEL_API void hel_ObjRotScaleUpdate(THandle Handle, u32 Angle, s32 ScaleX, s32 ScaleY);
HEL_API void hel_ObjRotScaleTransmit(void);
HEL_API HEL_INLINE u32 ATTR_WARNUNUSEDRESULT hel_ObjRotScaleGetFreeSlots(void);



//////////////////////////////////////////////////////////////////////////
// BG functions
//
// Implementation:
//		helBg.c
//////////////////////////////////////////////////////////////////////////


#define __hel_BgSetMode(Mode)		do { M_DISCNT_BGMODE_SET((Mode)); } while(FALSE)

#ifdef HEL_DEBUG
	HEL_API void hel_BgSetMode(u32 Mode);
#else
	#define hel_BgSetMode(Mode)		__hel_BgSetMode(Mode)
#endif


HEL_API void hel_BgSetMosaic(u32 BgNo, u32 Value);
HEL_API void hel_BgSetMosaicSize(u32 HSize, u32 VSize);
HEL_API void hel_BgSetPrio(u32 BgNo, u32 Prio);



//////////////////////////////////////////////////////////////////////////
// DMA functions
//
// Implementation:
//		helDma.c, helDmaImplementation.s
//////////////////////////////////////////////////////////////////////////

#define DMA_INCREMENT       0
#define DMA_DECREMENT       1
#define DMA_NOCHANGE        2
#define DMA_REPEAT_ON       1
#define DMA_REPEAT_OFF      0
#define DMA_IRQ_ON          1
#define DMA_IRQ_OFF         0


#define __hel_DmaCopy16(pDest, pSrc, SizeIn16BitWords)				__hel_DmaCopy16Implementation((void*)(pSrc), (void*)(pDest), SizeIn16BitWords)
#define __hel_DmaCopy16VBL(pDest, pSrc, SizeIn16BitWords)			__hel_DmaCopy16VBlankImplementation((void*)(pSrc), (void*)(pDest), SizeIn16BitWords)
#define __hel_DmaCopy32(pDest, pSrc, SizeIn32BitWords)				__hel_DmaCopy32Implementation((void*)(pSrc), (void*)(pDest), SizeIn32BitWords)
#define __hel_DmaSet16(pDest, SrcValue, SizeIn16BitWords)			__hel_DmaSet16Implementation(SrcValue, (void*)(pDest), SizeIn16BitWords)
#define __hel_DmaZeroMemory(pDest, SizeInBytes)						hel_DmaSet8((void*)(pDest), 0, SizeInBytes)


#ifdef HEL_DEBUG
	HEL_API void hel_DmaCopy16(void *pDest, const void *pSrc, u32 SizeIn16BitWords);
	HEL_API void hel_DmaCopy32(void *pDest, const void *pSrc, u32 SizeIn32BitWords);
	HEL_API void hel_DmaSet16(void *pDest, u16 SrcValue, u32 SizeIn16BitWords);
	HEL_API void hel_DmaZeroMemory(void *pDest, u32 SizeInBytes);
	HEL_API void hel_DmaCopy16VBL(void *pDest, const void *pSrc, u32 SizeIn16BitWords);
#else
	#define hel_DmaCopy16(pDest, pSrc, SizeIn16BitWords)    __hel_DmaCopy16(pDest, pSrc, SizeIn16BitWords)
	#define hel_DmaCopy32(pDest, pSrc, SizeIn16BitWords)    __hel_DmaCopy32(pDest, pSrc, SizeIn16BitWords)
	#define hel_DmaSet16(pDest, SrcValue, SizeIn16BitWords) __hel_DmaSet16(pDest, SrcValue, SizeIn16BitWords)
	#define hel_DmaZeroMemory(pDest, SizeInBytes)           __hel_DmaZeroMemory(pDest, SizeInBytes)
	#define hel_DmaCopy16VBL(pDest, pSrc, SizeIn16BitWords) __hel_DmaCopy16VBL(pDest, pSrc, SizeIn16BitWords)
#endif 


HEL_API void hel_DmaSet8(void *pDest, u8 SrcValue, u32 SizeInBytes);
HEL_API void hel_DmaSet32(void *pDest, u32 SrcValue, u32 SizeIn32BitWords);

// Do not directly use these functions. These are the implementation functions only,
// held most suitable for the GBA, having the parameters in the order the hardware
// wants them to keep it fast. Use the normal hel_Dma* function instead!
HEL_API void __hel_DmaCopy16Implementation(const void *pSrc, void *pDest, u32 SizeIn16BitWords);
HEL_API void __hel_DmaCopy16VBlankImplementation(const void *pSrc, void *pDest, u32 SizeIn16BitWords);
HEL_API void __hel_DmaCopy32Implementation(const void *pSrc, void *pDest, u32 SizeIn32BitWords);
HEL_API void __hel_DmaSet16Implementation(u16 SrcValue, void *pDest, u32 SizeIn16BitWords);



//////////////////////////////////////////////////////////////////////////
// Splash functions
//
// Implementation:
//		helSplash.c
//////////////////////////////////////////////////////////////////////////

/*!
\ingroup helSplashFlags
\hideinitializer
\brief  Button A

	This flag will enable the corresponding button
	to properly function with #SPLASH_FLAGS_WAITFORBUTTONPRESS and 
	#SPLASH_FLAGS_LEAVEONBUTTONPRESS. 

\note
	You can \p OR the \p SPLASH_FLAGS_BUTTON_* flags to enable multiple buttons.
\sa
	#hel_Splash
*/
#define SPLASH_FLAGS_BUTTON_A		BIT0

/*!
\ingroup helSplashFlags
\hideinitializer
\brief  Button B

	This flag will enable the corresponding button
	to properly function with #SPLASH_FLAGS_WAITFORBUTTONPRESS and 
	#SPLASH_FLAGS_LEAVEONBUTTONPRESS. 

\note
	You can \p OR the \p SPLASH_FLAGS_BUTTON_* flags to enable multiple buttons.
\sa
	#hel_Splash
*/
#define SPLASH_FLAGS_BUTTON_B		BIT1

/*!
\ingroup helSplashFlags
\hideinitializer
\brief  Button Select

	This flag will enable the corresponding button
	to properly function with #SPLASH_FLAGS_WAITFORBUTTONPRESS and 
	#SPLASH_FLAGS_LEAVEONBUTTONPRESS. 

\note
	You can \p OR the \p SPLASH_FLAGS_BUTTON_* flags to enable multiple buttons.
\sa
	#hel_Splash
*/
#define SPLASH_FLAGS_BUTTON_SELECT	BIT2

/*!
\ingroup helSplashFlags
\hideinitializer
\brief  Button Start

	This flag will enable the corresponding button
	to properly function with #SPLASH_FLAGS_WAITFORBUTTONPRESS and 
	#SPLASH_FLAGS_LEAVEONBUTTONPRESS. 

\note
	You can \p OR the \p SPLASH_FLAGS_BUTTON_* flags to enable multiple buttons.
\sa
	#hel_Splash
*/
#define SPLASH_FLAGS_BUTTON_START	BIT3

/*!
\ingroup helSplashFlags
\hideinitializer
\brief  Button Right direction

	This flag will enable the corresponding button
	to properly function with #SPLASH_FLAGS_WAITFORBUTTONPRESS and 
	#SPLASH_FLAGS_LEAVEONBUTTONPRESS. 

\note
	You can \p OR the \p SPLASH_FLAGS_BUTTON_* flags to enable multiple buttons.
\sa
	#hel_Splash
*/
#define SPLASH_FLAGS_BUTTON_RIGHT	BIT4

/*!
\ingroup helSplashFlags
\hideinitializer
\brief  Button Left direction

	This flag will enable the corresponding button
	to properly function with #SPLASH_FLAGS_WAITFORBUTTONPRESS and 
	#SPLASH_FLAGS_LEAVEONBUTTONPRESS. 

\note
	You can \p OR the \p SPLASH_FLAGS_BUTTON_* flags to enable multiple buttons.
\sa
	#hel_Splash
*/
#define SPLASH_FLAGS_BUTTON_LEFT	BIT5

/*!
\ingroup helSplashFlags
\hideinitializer
\brief  Button Up direction

	This flag will enable the corresponding button
	to properly function with #SPLASH_FLAGS_WAITFORBUTTONPRESS and 
	#SPLASH_FLAGS_LEAVEONBUTTONPRESS. 

\note
	You can \p OR the \p SPLASH_FLAGS_BUTTON_* flags to enable multiple buttons.
\sa
	#hel_Splash
*/
#define SPLASH_FLAGS_BUTTON_UP		BIT6

/*!
\ingroup helSplashFlags
\hideinitializer
\brief  Button Down direction

	This flag will enable the corresponding button
	to properly function with #SPLASH_FLAGS_WAITFORBUTTONPRESS and 
	#SPLASH_FLAGS_LEAVEONBUTTONPRESS. 

\note
	You can \p OR the \p SPLASH_FLAGS_BUTTON_* flags to enable multiple buttons.
\sa
	#hel_Splash
*/
#define SPLASH_FLAGS_BUTTON_DOWN	BIT7

/*!
\ingroup helSplashFlags
\hideinitializer
\brief  Button Right shoulder button

This flag will enable the corresponding button
to properly function with #SPLASH_FLAGS_WAITFORBUTTONPRESS and 
#SPLASH_FLAGS_LEAVEONBUTTONPRESS. 

\note
You can \p OR the \p SPLASH_FLAGS_BUTTON_* flags to enable multiple buttons.
\sa
#hel_Splash
*/
#define SPLASH_FLAGS_BUTTON_R		BIT8

/*!
\ingroup helSplashFlags
\hideinitializer
\brief  Button Left shoulder button

	This flag will enable the corresponding button
	to properly function with #SPLASH_FLAGS_WAITFORBUTTONPRESS and 
	#SPLASH_FLAGS_LEAVEONBUTTONPRESS. 

\note
	You can \p OR the \p SPLASH_FLAGS_BUTTON_* flags to enable multiple buttons.
\sa
	#hel_Splash
*/
#define SPLASH_FLAGS_BUTTON_L		BIT9

/*!
\ingroup helSplashFlags
\hideinitializer
\brief  All Buttons

	This flag will enable all buttons to properly function
	with #SPLASH_FLAGS_WAITFORBUTTONPRESS and #SPLASH_FLAGS_LEAVEONBUTTONPRESS. 

\sa
	#hel_Splash
*/
#define SPLASH_FLAGS_BUTTON_ALL		(SPLASH_FLAGS_BUTTON_A | SPLASH_FLAGS_BUTTON_B | SPLASH_FLAGS_BUTTON_SELECT |	\
									SPLASH_FLAGS_BUTTON_START | SPLASH_FLAGS_BUTTON_RIGHT | SPLASH_FLAGS_BUTTON_LEFT | SPLASH_FLAGS_BUTTON_UP	| \
									SPLASH_FLAGS_BUTTON_DOWN	|	SPLASH_FLAGS_BUTTON_R | SPLASH_FLAGS_BUTTON_L)

/*!
\ingroup helSplashFlags
\hideinitializer
\brief  Wait for button press

	This flag will display the splash logo until a button has pressed. 

\sa
	#hel_Splash
*/
#define SPLASH_FLAGS_WAITFORBUTTONPRESS		BIT10

/*!
\ingroup helSplashFlags
\hideinitializer
\brief  Leave on button press

	This flag make the splash screen leave when a button has pressed.
	The splash screen will be correctly blended out.

\sa
	#hel_Splash
*/
#define SPLASH_FLAGS_LEAVEONBUTTONPRESS		BIT11

/*!
\ingroup helSplashFlags
\hideinitializer
\brief  Default splash screen flags

	The #SPLASH_FLAGS_DEFAULT define represents the common
	used splash screen flags. All buttons are enabled and
	it will leave on button press.

\sa
	#hel_Splash
*/
#define SPLASH_FLAGS_DEFAULT				(SPLASH_FLAGS_BUTTON_ALL | SPLASH_FLAGS_LEAVEONBUTTONPRESS)

HEL_API void hel_SplashInit(void *pBuffer);
HEL_API void hel_SplashQuit();
HEL_API void hel_Splash(const void *pBitmap, const u16 *pPalette, u16 InBlendColor, u16 OutBlendColor, u32 BlendDelay, u32 DisplayDelay, u32 ContinueDelay, u32 CompressionType, u32 Flags);



//////////////////////////////////////////////////////////////////////////
// Interrupt functions
//
// Implementation:
//		helInt.c
//////////////////////////////////////////////////////////////////////////

typedef void (*PInterruptProc)(void);

#define __hel_IntrAcknowledge(InterruptType) do { R_INTREQ |= (1 << (InterruptType)); *((volatile u16*)0x3fffff8) |= (1 << (InterruptType)); } while(0)

#ifdef HEL_DEBUG
	HEL_API void hel_IntrAcknowledge(u32 InterruptType);
#else
	#define hel_IntrAcknowledge(InterruptType)	__hel_IntrAcknowledge(InterruptType)
#endif

HEL_API void hel_IntrStartHandler(u32 InterruptType, void *pInterruptHandler);
HEL_API void hel_IntrStopHandler(u32 InterruptType);
HEL_API void hel_IntrUpdateHandler(u32 InterruptType, void *pInterruptHandler);
HEL_API void hel_IntrStopAll(void);
HEL_API u32 ATTR_WARNUNUSEDRESULT hel_IntrIsEnabled(u32 InterruptType);
HEL_API u32 ATTR_WARNUNUSEDRESULT hel_IntrGetType(void *pInterruptProc);



//////////////////////////////////////////////////////////////////////////
// Bitmap-Mode functions
//
// Implementation:
//		helBmp.c
//////////////////////////////////////////////////////////////////////////

#define __hel_BmpGetBackBuffer()		(F_DISCNT_BUFSEL_GET)?(u16*)MEM_VRAM:(u16*)(MEM_VRAM + 0xA000)
#define __hel_BmpHasBackBuffer()		(F_DISCNT_BGMODE_GET > 3)
#define __hel_BmpIsHighColor()			((F_DISCNT_BGMODE_GET==3)||(F_DISCNT_BGMODE_GET==5))
#define __hel_BmpFlipBackBuffer()		do { R_DISCNT ^= BIT4; } while(0)

#ifdef HEL_DEBUG
	HEL_API u16 *hel_BmpGetBackBuffer(void);
	HEL_API u32 ATTR_PURE hel_BmpHasBackBuffer(void);
	HEL_API u32 ATTR_PURE hel_BmpIsHighColor(void);
	HEL_API void hel_BmpFlipBackBuffer(void);
#else
	#define hel_BmpGetBackBuffer()		__hel_BmpGetBackBuffer()
	#define hel_BmpHasBackBuffer()		__hel_BmpHasBackBuffer()
	#define hel_BmpIsHighColor()		__hel_BmpIsHighColor()
	#define hel_BmpFlipBackBuffer()		__hel_BmpFlipBackBuffer()
#endif

HEL_API void hel_BmpClear(u16 Color);
HEL_API void hel_BmpLoad(const void *pSrc);
HEL_API void hel_BmpLoadUnComp(const void *pSrc, u32 CompressionType);


//////////////////////////////////////////////////////////////////////////
// Debug Assertion Functions
//
// Implementation:
//		helAssert.c, helAssertFont.c
//////////////////////////////////////////////////////////////////////////

// Following structures must be available in non debug mode too,
// to make sure not to break user code.

// DebugAssertInfo is passed to the 'OnAssert'
// function in case one has been specified. It contains
// all the data the normal Assert function has too.
typedef struct _TDebugAssertInfo 
{
	const char *pExpression;
	const char *pFilename;
	const char *pFunctionName;
	u32 LineNo;
	const char *pMessage;
	u32 nCallStack;
	const char **ppCallStack;
}TDebugAssertInfo, *PDebugAssertInfo;

typedef void (*PDebugAssertFunc)(const TDebugAssertInfo *pInfo);
typedef int (*PDebugMessageFunc)(const char *pText, ...);


#if defined(HEL_DEBUG) || defined(HEL_CHECKED)

	// Information about one character from
	// the assertion debug system
	typedef struct _TAssertCharInfo
	{
		u8 Width;										// Width of character in pixels
		u8 PitchY;									// Amount in pixel to move the character downwards when drawing
		const char CharData[5][8];	// The character itself
	}TAssertCharInfo, *PAssertCharInfo;

#endif


/*!
\ingroup PageDebug
\hideinitializer
\brief Static assertion evaluation mechanism

	The #HEL_SASSERT macro can be used to evaluate an expression at compile time.

\param[in]
	expression		Expression that evaluates to \c TRUE or \c FALSE

\note
	#HEL_SASSERT works in all HEL versions, \p HEL_DEBUG, \p HEL_CHECKED and in release mode.
*/
#define HEL_SASSERT(expression)\
	{ enum { HEL_STATIC_ASSERTION_FAILED = 1/(expression) }; }


#if !defined(HEL_BUILDWITHOUTASSERT) && (defined(HEL_DEBUG) || defined(HEL_CHECKED))

/*!
\ingroup PageDebug
\hideinitializer
\brief Assertion evaluation mechanism

	The #HEL_ASSERT mechanism evaluates an expression and, when the result is \p FALSE, 
	displays an error screen and aborts the program execution until you press
	a specific button. The error-screen displays several informations where in the
	source the assertion failed. 
	This includes:
		\li	Filename
		\li Line
		\li Function
		\li Expression
		\li Description

\par Assertion Screen
\image html assert_01.png
\image latex assert_01.eps

\par Assertion Screen displaying the call stack
\image html assert_02.png
\image latex assert_02.eps

\param[in]
	expression		Expression that evaluates to \c TRUE or \c FALSE
\param[in]
	message				Message to display when expression is \c FALSE

\note
	The #HEL_ASSERT macro is typically used to identify logic errors during 
	program development by implementing the expression argument to evaluate to \p FALSE
	only when the program is operating incorrectly. 

	#HEL_ASSERT works only in \p HEL_DEBUG and \p HEL_CHECKED versions.
\sa
	#hel_DebugSetOnAssert, \ref PageLibraries
*/
	#define HEL_ASSERT(expression, message) {\
		if(!(expression))\
		{\
			static const char* const Expression=#expression;\
			static const char* const Message=message;\
			static const char* const File=__FILE__;\
			hel_DebugAssert(Expression, File, __FUNCTION__, __LINE__, Message);\
		}} 

/*!
\ingroup PageDebug
\brief Specify a function that gets called before an assertion takes place

	You can specify a function that gets called before an assertion takes place. This
	can be helpful if you want to output your engine stats or whatever when something
	went wrong. Use the #hel_DebugSetOnAssert function for this.

\param[in]
	pFunc			A pointer to the function that should be called before an assertion takes place
*/
	HEL_API void ATTR_NOINSTRUMENT hel_DebugSetOnAssert(PDebugAssertFunc pFunc);

	HEL_API void ATTR_NORETURN ATTR_NOINSTRUMENT hel_DebugAssert(const char* const pExpression, const char* const pFilename, const char* const pFunction, u32 LineNo, const char* const pMessage, ...);

#else

	#define HEL_ASSERT(expression, message)
	#define hel_DebugSetOnAssert(pFunc)

#endif // HEL_DEBUG



//////////////////////////////////////////////////////////////////////////
// Debug Message Functions
//
// Implementation:
//		helDebugNoChashMessage.c, helDebugNoCashMessageImplementation.s,
//		helDebugVbaMessage.c
//////////////////////////////////////////////////////////////////////////

#define DEBUG_MESSAGETYPE_VBA		0
#define DEBUG_MESSAGETYPE_NOCASH	1

#ifdef HEL_DEBUG_MESSAGE_ENABLED

	HEL_API void hel_DebugSetMessageType(u32 MessageType);

	// Do not use these functions, use HEL_DEBUG_MSG instead
	HEL_API PDebugMessageFunc __hel_DebugMessageFunc;
	HEL_API char __hel_DebugBuffer[];
	HEL_API int ATTR_NOINSTRUMENT __hel_DebugNoCashMessage(const char *pText, ...);
	HEL_API int ATTR_NOINSTRUMENT __hel_DebugVbaMessage(const char *pText, ...);
#else

	#define hel_DebugSetMessageType(MessageType)

#endif


//////////////////////////////////////////////////////////////////////////
// Call Stack Trace Functions
//
// Implementation:
//		helDebugCallStackTrace.c
//////////////////////////////////////////////////////////////////////////

#ifdef HEL_DEBUG_CALLSTACKTRACE_ENABLED

	HEL_API const char* ATTR_NOINSTRUMENT hel_DebugCallStackTraceGetFuntionName(u32 Level);
	HEL_API u32 ATTR_NOINSTRUMENT hel_DebugCallStackTraceGetFuntionAddr(u32 Level);
	HEL_API u32 ATTR_NOINSTRUMENT hel_DebugCallStackTraceGetCurrentLevel(void);

#else

	#define hel_DebugCallStackTraceGetFuntionName(Level)		NULL
	#define hel_DebugCallStackTraceGetFuntionAddr(Level)		0
	#define hel_DebugCallStackTraceGetCurrentLevel()			0

#endif

//////////////////////////////////////////////////////////////////////////
// System Functions
//
// Implementation:
//		helSys.c
//////////////////////////////////////////////////////////////////////////

HEL_API void hel_SysSetPrefetch(u32 Value);



//////////////////////////////////////////////////////////////////////////
// Window Functions
//
// Implementation:
//		helWin.c
//////////////////////////////////////////////////////////////////////////

// R_WIN1Y is wrong in HAM 2.8, so we undefine it
// and define a new one which points to the 
// correct memory location!
#undef  R_WIN1Y
#define R_WIN1Y		*(volatile u16 *)(MEM_IO + 0x46)

#define WIN_FX		(BIT5)
#define WIN_ALL		(WIN_BG0 | WIN_BG1 | WIN_BG2 | WIN_BG3 | WIN_OBJ)
#define WIN_NONE	(0)
#define WIN_SELECT(bg0, bg1, bg2, bg3, obj) ((bg0) | ((bg1) << WIN_BG1) | ((bg2) << WIN_BG2) | ((bg3) << WIN_BG3) | ((obj) << WIN_OBJ))


typedef struct _TWinInfo 
{
	u16 WinH[2];		// Window 0/1 Horizontal Dimensions
	u16 WinV[2];		// Window 0/1 Vertical Dimensions
	u16 WinIn;			// Control of Inside of Window(s)
	u16 WinOut;			// Control of Outside of Windows & Inside of OBJ Window
}TWinInfo, *PWinInfo;


#define __hel_WinCreate(WinNo, Left, Top, Right, Bottom, InsideFlags, OutsideFlags) hel_WinCreateEx(WinNo, Left, Top, Right, Bottom, InsideFlags, OutsideFlags, FALSE)
#define __hel_WinSetRegion(WinNo, Left, Top, Right, Bottom) { __hel_WinInfo.WinH[WinNo] = ((Left)<<8) | (Right); __hel_WinInfo.WinV[WinNo] = ((Top)<<8) | (Bottom); }
#define __hel_WinDelete(WinNo)                              __hel_WinHide(WinNo);
#define __hel_WinSetInsideFlags(WinNo, InsideFlags)         { __hel_WinInfo.WinIn &= (0xff00>>((WinNo)<<3)); __hel_WinInfo.WinIn |= ((InsideFlags)<<((WinNo)<<3)); }
#define __hel_WinSetOutsideFlags(WinNo, OutsideFlags)       { __hel_WinInfo.WinOut &= (0xff00>>((WinNo)<<3)); __hel_WinInfo.WinOut |= (OutsideFlags); }
#define __hel_WinShow(WinNo)                                { R_DISCNT |= BIT(13+(WinNo)); }
#define __hel_WinHide(WinNo)                                { R_DISCNT &= BIT_NOT(13+(WinNo)); }
#define __hel_WinSetVisible(WinNo, Value)                   { R_DISCNT &= BIT_NOT(13+(WinNo)); R_DISCNT |= ((Value) << BIT(13+(WinNo))); }


#ifdef HEL_DEBUG
  HEL_API void hel_WinCreate(u8 WinNo, u8 Left, u8 Top, u8 Right, u8 Bottom, u8 InsideFlags, u8 OutsideFlags);
  HEL_API void hel_WinDelete(u8 WinNo);
  HEL_API void hel_WinSetInsideFlags(u8 WinNo, u8 InsideFlags);
  HEL_API void hel_WinSetOutsideFlags(u8 WinNo, u8 OutsideFlags);
  HEL_API void hel_WinSetRegion(u8 WinNo, u8 Left, u8 Top, u8 Right, u8 Bottom);
  HEL_API void hel_WinShow(u8 WinNo);
  HEL_API void hel_WinHide(u8 WinNo);
  HEL_API void hel_WinSetVisible(u8 WinNo, u8 Value);
#else
  #define hel_WinCreate(WinNo, Left, Top, Right, Bottom, InsideFlags, OutsideFlags) __hel_WinCreate(WinNo, Left, Top, Right, Bottom, InsideFlags, OutsideFlags)
  #define hel_WinDelete(WinNo)                                __hel_WinDelete(WinNo)
  #define hel_WinSetInsideFlags(WinNo, InsideFlags)           __hel_WinSetInsideFlags(WinNo, InsideFlags)
  #define hel_WinSetOutsideFlags(WinNo, OutsideFlags)         __hel_WinSetOutsideFlags(WinNo, OutsideFlags)
  #define hel_WinSetRegion(WinNo, Left, Top, Right, Bottom)   __hel_WinSetRegion(WinNo, Left, Top, Right, Bottom)
  #define hel_WinShow(WinNo)                                  __hel_WinShow(WinNo)
  #define hel_WinHide(WinNo)                                  __hel_WinHide(WinNo)
  #define hel_WinSetVisible(WinNo, Value)                     __hel_WinSetVisible(WinNo, Value)
#endif

HEL_API void hel_WinInit(void);
HEL_API void hel_WinQuit(void);
HEL_API void hel_WinCreateEx(u8 WinNo, u8 Left, u8 Top, u8 Right, u8 Bottom, u8 InsideFlags, u8 OutsideFlags, u8 SkipOutsideFlags);
HEL_API HEL_INLINE void hel_WinTransmit(void);

// do not use this variable, use the accessors macros instead
HEL_API TWinInfo __hel_WinInfo;



//////////////////////////////////////////////////////////////////////////
// Special Effect Functions
//
// Implementation:
//		helFx.c
//////////////////////////////////////////////////////////////////////////

#define FX_LAYER_BG0   BIT0
#define FX_LAYER_BG1   BIT1
#define FX_LAYER_BG2   BIT2
#define FX_LAYER_BG3   BIT3
#define FX_LAYER_OBJ   BIT4
#define FX_LAYER_BD    BIT5
#define FX_LAYER_ALL   0x3f
#define FX_LAYER_NONE  0x00

typedef struct ATTR_PACKED _TEffectInfo 
{
	u16 Control;		// Represents R_BLDCNT
	u16 Alpha;			// Represents R_BLDALPHA
	u16 Brightness;		// Represents R_BLDY
} TEffectInfo, *PEffectInfo;

#define __hel_FxSetMode(SourceLayer, TargetLayer, FxMode)			__hel_EffectInfo.Control = (SourceLayer) | ((FxMode)<<6) | ((TargetLayer)<<8)
#define __hel_FxSetAlphaLevel(SourceIntensity, TargetIntensity)		__hel_EffectInfo.Alpha = (SourceIntensity) | ((TargetIntensity)<<8)
#define __hel_FxSetBrightnessLevel(Intensity)						__hel_EffectInfo.Brightness = (Intensity)

#ifdef HEL_DEBUG
	HEL_API void hel_FxSetMode(u32 SourceLayer, u32 TargetLayer, u32 FxMode);
	HEL_API void hel_FxSetAlphaLevel(u32 SourceIntensity, u32 TargetIntensity);
	HEL_API void hel_FxSetBrightnessLevel(u32 Intensity);
#else
	#define hel_FxSetMode(SourceLayer, TargetLayer, FxMode)			__hel_FxSetMode(SourceLayer, TargetLayer, FxMode)
	#define hel_FxSetAlphaLevel(SourceIntensity, TargetIntensity)	__hel_FxSetAlphaLevel(SourceIntensity, TargetIntensity)
	#define hel_FxSetBrightnessLevel(Intensity)						__hel_FxSetBrightnessLevel(Intensity)            
#endif


HEL_API void hel_FxInit(void);
HEL_API void hel_FxQuit(void);

// helFxTransmit.s
HEL_API void hel_FxTransmit(void);

// do not use this variable, use the accessor macros instead
HEL_API TEffectInfo __hel_EffectInfo;


//////////////////////////////////////////////////////////////////////////
// BIOS Functions
//
// Implementation:
//		helSwi.c
//////////////////////////////////////////////////////////////////////////

#define COMPRESSION_TYPE_NONE				0
#define COMPRESSION_TYPE_LZ77VRAM			1
#define COMPRESSION_TYPE_LZ77WRAM			2
#define COMPRESSION_TYPE_RLEVRAM			3
#define COMPRESSION_TYPE_RLEWRAM			4

typedef void (*PDecompressFunc)(void *pSrc, void *pDst);


#define __hel_SwiUnComp(pDst, pSrc, CompressionType)	__helSwiDecompressJumpTable[CompressionType]((void*)(pSrc), (void*)(pDst))
#define __hel_SwiLZ77UnCompVram(pDst, pSrc)				__hel_SwiLZ77UnCompVramImplementation((void*)(pSrc), (void*)(pDst))
#define __hel_SwiLZ77UnCompWram(pDst, pSrc)				__hel_SwiLZ77UnCompWramImplementation((void*)(pSrc), (void*)(pDst))
#define __hel_SwiRLUnCompVram(pDst, pSrc)				__hel_RLUnCompVramImplementation((void*)(pSrc), (void*)(pDst))
#define __hel_SwiRLUnCompWram(pDst, pSrc)				__hel_RLUnCompWramImplementation((void*)(pSrc), (void*)(pDst))
#define __hel_SwiCpuFastCopy(pDst, pSrc, LengthInWords)	__hel_SwiCpuFastCopyImplementation((const void*)pSrc, (void*)pDst, LengthInWords)

// To stay compatible with CLib memcpy as well 
// as DMA functions: First parameter is pDst, pSrc is second!
#ifdef HEL_DEBUG
  HEL_API void hel_SwiUnComp(void *pDst, const void *pSrc, u32 CompressionType);
  HEL_API void hel_SwiLZ77UnCompVram(void *pDst, const void *pSrc);
  HEL_API void hel_SwiLZ77UnCompWram(void *pDst, const void *pSrc);
  HEL_API void hel_SwiRLUnCompVram(void *pDst, const void *pSrc);
  HEL_API void hel_SwiRLUnCompWram(void *pDst, const void *pSrc);
  HEL_API void ATTR_NOINLINE ATTR_NOINSTRUMENT hel_SwiCpuFastCopy(void *pDst, const void* pSrc, u32 LengthInWords);
#else
  #define hel_SwiUnComp(pDst, pSrc, CompressionType)	__hel_SwiUnComp(pDst, pSrc, CompressionType)
  #define hel_SwiLZ77UnCompVram(pDst, pSrc)				__hel_SwiLZ77UnCompVram(pDst, pSrc)
  #define hel_SwiLZ77UnCompWram(pDst, pSrc)				__hel_SwiLZ77UnCompWram(pDst, pSrc)
  #define hel_SwiRLUnCompVram(pDst, pSrc)				__hel_SwiRLUnCompVram(pDst, pSrc)
  #define hel_SwiRLUnCompWram(pDst, pSrc)				__hel_SwiRLUnCompVram(pDst, pSrc)
  #define hel_SwiCpuFastCopy(pDst, pSrc, LengthInWords)	__hel_SwiCpuFastCopy(pDst, pSrc, LengthInWords)
#endif

HEL_API s32  ATTR_NOINLINE ATTR_NOINSTRUMENT hel_SwiMod(s32 Number, s32 Denom);
HEL_API s32  ATTR_NOINLINE ATTR_NOINSTRUMENT hel_SwiDiv(s32 Number, s32 Denom);
HEL_API void ATTR_NOINLINE ATTR_NOINSTRUMENT hel_SwiVBlankIntrWait(void);
HEL_API void ATTR_NOINLINE ATTR_NOINSTRUMENT hel_SwiHalt(void);
HEL_API void ATTR_NOINLINE ATTR_NOINSTRUMENT hel_SwiSoftReset(u32 ReturnAddr);

// Do not use these functions, use the regular hel_Swi* functions instead!
//
// pSrc and pDst parameters are ordered differently compared to the
// regular hel_Swi* functions, in order to have them closer to what
// the hardware expects. 
HEL_API const PDecompressFunc __helSwiDecompressJumpTable[];
HEL_API void ATTR_NOINLINE __hel_SwiLZ77UnCompVramImplementation(void *pSrc, void *pDst);
HEL_API void ATTR_NOINLINE __hel_SwiLZ77UnCompWramImplementation(void *pSrc, void *pDst);
HEL_API void ATTR_NOINLINE __hel_RLUnCompVramImplementation(void *pSrc, void *pDst);
HEL_API void ATTR_NOINLINE __hel_RLUnCompWramImplementation(void *pSrc, void *pDst);
HEL_API void ATTR_NOINLINE ATTR_NOINSTRUMENT __hel_SwiCpuFastCopyImplementation(const void* pSrc, void *pDst, u32 LengthInWords);


//////////////////////////////////////////////////////////////////////////
// Math Functions
//
// Implementation:
//		helMath.c, helMathExtractDigits.s
//////////////////////////////////////////////////////////////////////////

HEL_API void hel_MathExtractDigits(u32 Value, u8 *pOut, u32 nDigits);

	

#endif // HEL2_H
